/* command.cpp  -- Command class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      command.cpp
 * @author    Bruno ANSELME
 * @brief     Command class definition
 * @details   Command types, mouse types and meta types definitions.\n
 *            Types translation to text and text to type.
 */

#include "guicontrol.h"
#include "command.h"

static QMap<Command::comType,   QString>    type_name;          // Labels for Command::comType
static QMap<Command::metaType,  QString>    meta_name;          // Labels for Command::metaType
static QMap<Command::mouseType, QString>    mouse_name;         // Labels for Command::mouseType
static QMap<QString,    Command::comType>   type_key;           // Reversed type_name
static QMap<QString,    Command::metaType>  meta_key;           // Reversed meta_name
static QMap<QString,    Command::mouseType> mouse_key;          // Reversed mouse_name

void Command::initCommands() {
    if (type_name.isEmpty()) {
        type_name[Meta]            = QString("Meta");
        type_name[Key]             = QString("Key");
        type_name[Mouse]           = QString("Mouse");
        type_name[Text]            = QString("Text");
        type_name[Exec]            = QString("Exec");
        QMapIterator<comType, QString> i(type_name);
        while (i.hasNext()) { i.next(); type_key[i.value()] = i.key(); }
    }
    if (meta_name.isEmpty()) {
        meta_name[Quit]            = QString("Quit");
        meta_name[Stop]            = QString("Stop");
        meta_name[Suspend]         = QString("Suspend");
        meta_name[Resume]          = QString("Resume");
        meta_name[Repeat]          = QString("Repeat");
        meta_name[Vocabulary]      = QString("Vocabulary");
        meta_name[CommandSets]     = QString("CommandSets");
        meta_name[Settings]        = QString("Settings");
        meta_name[Reminder]        = QString("Reminder");
        meta_name[Raise]           = QString("Raise");
        meta_name[Minimize]        = QString("Minimize");
        meta_name[MouseShow]       = QString("MouseShow");
        QMapIterator<metaType, QString> i(meta_name);
        while (i.hasNext()) { i.next(); meta_key[i.value()] = i.key(); }
    }
    if (mouse_name.isEmpty()) {
        mouse_name[Left]           = QString("Click left");
        mouse_name[Middle]         = QString("Click middle");
        mouse_name[Right]          = QString("Click right");
        mouse_name[DownLeft]       = QString("Press left");
        mouse_name[DownMiddle]     = QString("Press middle");
        mouse_name[DownRight]      = QString("Press right");
        mouse_name[UpLeft]         = QString("Release left");
        mouse_name[UpMiddle]       = QString("Release middle");
        mouse_name[UpRight]        = QString("Release right");
        mouse_name[WheelUp]        = QString("Wheel up");
        mouse_name[WheelDown]      = QString("Wheel down");
        mouse_name[MoveUp]         = QString("Move up");
        mouse_name[MoveDown]       = QString("Move down");
        mouse_name[MoveLeft]       = QString("Move left");
        mouse_name[MoveRight]      = QString("Move right");
        mouse_name[MoveTo]         = QString("Move to");
        mouse_name[GlideUp]        = QString("Glide up");
        mouse_name[GlideDown]      = QString("Glide down");
        mouse_name[GlideLeft]      = QString("Glide left");
        mouse_name[GlideRight]     = QString("Glide right");
        mouse_name[GlideUpLeft]    = QString("Glide up left");
        mouse_name[GlideUpRight]   = QString("Glide up right");
        mouse_name[GlideDownLeft]  = QString("Glide down left");
        mouse_name[GlideDownRight] = QString("Glide down right");
        QMapIterator<mouseType, QString> i(mouse_name);
        while (i.hasNext()) { i.next(); mouse_key[i.value()] = i.key(); }
    }
}

//----------------------------
Command::Command(comType typ) :   cmdtype(typ) { }

Command *Command::createKeyCommand(const QString &keyseq) {
    Command *cmd    = new Command(Key);
    cmd->setQtKey(keyseq);
    cmd->qt2keysym();
    return cmd;
}

Command *Command::createMetaCommand(metaType mc){
    Command *cmd    = new Command(Meta);
    cmd->setMetaType(mc);
    cmd->setQtKey(QString(meta_name[cmd->getMetaType()]));
    cmd->setKeySym(cmd->getQtKey());
    return cmd;
}

Command *Command::createMouseCommand(mouseType ms){
    Command *cmd    = new Command(Mouse);
    cmd->setMouseType(ms);
    cmd->setQtKey(QString(mouse_name[cmd->getMouseType()]));
    cmd->setKeySym(cmd->getQtKey());
    return cmd;
}

Command *Command::createTextCommand(const QString &textToSend) {
    Command *cmd = new Command(Text);
    QString cmdtxt = removeQuotes(textToSend);
    cmd->setQtKey(cmdtxt);
    cmd->setKeySym(cmd->getQtKey());
    return cmd;
}

Command *Command::createExecCommand(const QString &application) {
    Command *cmd = new Command(Exec);
    cmd->setQtKey(application);
    cmd->setKeySym(cmd->getQtKey());
    return cmd;
}

QString Command::addQuotes(const QString s) {
    QRegExp re("^\".*\"$");
    if (re.exactMatch(s)) {
        return s;
    } else {
        return QString("\"%1\"").arg(s);
    }
}

QString Command::removeQuotes(const QString s) {
    QRegExp re("^\"(.*)\"$");
    QStringList     pieces;
    if (re.exactMatch(s)) {
        pieces      = re.capturedTexts();
        return pieces[1];
    }
    return s;
}

const QString &Command::getTypeName()               { return type_name[cmdtype];    }
const QString &Command::getMetaName()               { return meta_name[meta];       }
const QString &Command::getMouseName()              { return mouse_name[mouse];     }

const QString &Command::getTypeName(comType t)      { return type_name[t];          }
const QString &Command::getMetaName(metaType m)     { return meta_name[m];          }
const QString &Command::getMouseName(mouseType m)   { return mouse_name[m];         }

QStringList Command::getTypesNames() {
    QStringList lst;
    QMapIterator<comType, QString> i(type_name);
    while (i.hasNext()) { i.next(); lst.append(i.value()); }
    return lst;
}

QStringList Command::getMetaNames() {
    QStringList lst;
    QMapIterator<metaType, QString> i(meta_name);
    while (i.hasNext()) { i.next(); lst.append(i.value()); }
    return lst;
}

QStringList Command::getMouseNames() {
    QStringList lst;
    QMapIterator<mouseType, QString> i(mouse_name);
    while (i.hasNext()) { i.next(); lst.append(i.value()); }
    return lst;
}

Command::comType     Command::typeFromName(QString name)    { return type_key[name];   }
Command::metaType    Command::metaFromName(QString name)    { return meta_key[name];   }
Command::mouseType   Command::mouseFromName(QString name)   { return mouse_key[name];  }

void Command::qt2keysym() {     // Convert QKeySequence to KeySyms sequence
    if (cmdtype == Key) {
        cmdkeysym   = cmdqt;
        cmdkeysym   = guiKeySequence(cmdqt);
    }
}
