/* command.h  -- Command class definition for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COMMAND_H
#define COMMAND_H

#include <QObject>
#include <QFile>
#include <QMap>
#include <QHash>

#define SEQ_TO_NATIVE(s)    QKeySequence::fromString(s, QKeySequence::PortableText).toString(QKeySequence::NativeText)
#define SEQ_TO_PORTABLE(s)  QKeySequence::fromString(s, QKeySequence::NativeText).toString(QKeySequence::PortableText)

class Command {
public:
    enum    comType     { Meta,             //!< Meta type
                          Key,              //!< Key sequence type
                          Mouse,            //!< Mouse control type
                          Text,             //!< Text input type
                          Exec              //!< Execute command type
                        };
    enum    metaType    { Quit,             //!< Quit application
                          Stop,             //!< Stop listening
                          Suspend,          //!< Suspend listening
                          Resume,           //!< Resume listening
                          Repeat,           //!< Repeat last valid utterance (except meta commands)
                          Vocabulary,       //!< Open vocabulary dialog
                          CommandSets,      //!< Open commands sets dialog
                          Settings,         //!< Open settings dialog
                          Reminder,         //!< Open reminder window
                          Raise,            //!< Raise application window (deminimize, try to bring to front)
                          Minimize,         //!< Minimize application window
                          MouseShow         //!< Run animation to locate mouse pointer
                        };
    enum    mouseType   { Left,             //!< Click left button
                          Middle,           //!< Click middle button
                          Right,            //!< Click right button
                          WheelUp,          //!< Mouse wheel up
                          WheelDown,        //!< Mouse wheel down
                          DownLeft,         //!< Press left button
                          DownMiddle,       //!< Press middle button
                          DownRight,        //!< Press right button
                          UpLeft,           //!< Release left button
                          UpMiddle,         //!< Release middle button
                          UpRight,          //!< Release right button
                          MoveLeft,         //!< Mouse relative move left
                          MoveRight,        //!< Mouse relative move right
                          MoveUp,           //!< Mouse relative move up
                          MoveDown,         //!< Mouse relative move down
                          MoveTo,           //!< Mouse absolute move to
                          GlideLeft,        //!< Mouse glide left
                          GlideRight,       //!< Mouse glide right
                          GlideUp,          //!< Mouse glide up
                          GlideDown,        //!< Mouse glide down
                          GlideUpLeft,      //!< Mouse glide up left
                          GlideUpRight,     //!< Mouse glide up right
                          GlideDownLeft,    //!< Mouse glide down left
                          GlideDownRight    //!< Mouse glide down right
                        };
public:
    Command(comType typ);                                                       //!< Must never be called. Use static methods below
    static          Command    *createKeyCommand(const QString &keyseq);        //!< Key command constructor
    static          Command    *createMetaCommand(metaType mc);                 //!< Meta command constructor
    static          Command    *createMouseCommand(mouseType ms);               //!< Mouse command constructor
    static          Command    *createTextCommand(const QString &textToSend);   //!< Text command constructor
    static          Command    *createExecCommand(const QString &application);  //!< Exec command constructor
    static          void        initCommands();                                 //!< Run only once from CommandSet constructor

                    comType     getType()                       { return cmdtype;   }   //!< Return command type
            const   QString    &getKeySym()                     { return cmdkeysym; }   //!< Return KeySym key sequence string
            const   QString    &getQtKey()                      { return cmdqt;     }   //!< Return Qt key sequence string
            const   QString    &getExec()                       { return cmdqt;     }   //!< Return execution string
            const   QString    &getText()                       { return cmdqt;     }   //!< Return text input string
                    metaType    getMetaType()                   { return meta;      }   //!< Return meta type
                    mouseType   getMouseType()                  { return mouse;     }   //!< Return mouse type
            const   QString    &getTypeName();                                          //!< Return object's type string
            const   QString    &getMetaName();                                          //!< Return object's meta string
            const   QString    &getMouseName();                                         //!< Return object's mouse string
    static  const   QString    &getTypeName(comType t);                                 //!< Return type string from command type
    static  const   QString    &getMetaName(metaType m);                                //!< Return meta string from meta type
    static  const   QString    &getMouseName(mouseType m);                              //!< Return mouse string from mouse type

    static          QStringList getTypesNames();                                        //!< Return type names string list
    static          QStringList getMetaNames();                                         //!< Return meta names string list
    static          QStringList getMouseNames();                                        //!< Return mouse names string list

    static          comType     typeFromName(QString name);                             //!< Return type from type name
    static          metaType    metaFromName(QString name);                             //!< Return meta from meta name
    static          mouseType   mouseFromName(QString name);                            //!< Return mouse from mouse name

    static          QString     addQuotes(const QString s);                             //!< Add quotes around string
    static          QString     removeQuotes(const QString s);                          //!< Remove quotes around string

                    void        setQtKey(const QString &str)    { cmdqt     = str;  }   //!< Set Qt key sequence string
                    void        setKeySym(const QString &str)   { cmdkeysym = str;  }   //!< Set KeySym key sequence string
                    void        setMetaType(metaType m)         { meta      = m;    }   //!< Set meta type
                    void        setMouseType(mouseType m)       { mouse     = m;    }   //!< Set mouse type
                    void        qt2keysym();                                            //!< Convert Qt key sequence to KeySym key sequence
private:
    comType     cmdtype;
    QString     cmdkeysym;
    QString     cmdqt;      // Contains Qt Shortcut when comType==Key, text to send when comType==Text, command line when comType==Exec
    metaType    meta;
    mouseType   mouse;
};

#endif // COMMAND_H
