/* commandset.cpp  -- CommandSet class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      commandset.cpp
 * @author    Bruno ANSELME
 * @brief     CommandSet class definition
 * @details   Build command set from file.
 */

#include <QTextStream>
#include <QKeySequence>
#include <QFileInfo>
#include "commandset.h"

//----------------------------
CommandSet::CommandSet(QString fname) : QObject(),
    filename(fname),
    commands()
{
    Command::initCommands();
    loadCommands();
    QFileInfo   info(fname);
    name    = info.baseName();
}

void CommandSet::loadCommands() {               // Load commands and utterances from file.
    commands.clear();                           // This file is created from GUI by EditSet::accept() (editset.cpp)
    QFile file(filename);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;
    QTextStream f(&file);
    QString     cmd;
    QStringList slist;
    while (f.readLineInto(&cmd)) {              // Read command utterance
        slist = cmd.split(QString(" : "));
        Command::comType typ = static_cast<Command::comType>(slist.at(0).toInt());
        switch (typ) {
        case Command::Meta:
            commands[slist.at(1)]    = Command::createMetaCommand(static_cast<Command::metaType>(slist.at(2).toInt()));
            break;
        case Command::Key:
            commands[slist.at(1)]    = Command::createKeyCommand(slist.at(2));
            break;
        case Command::Mouse:
            commands[slist.at(1)]    = Command::createMouseCommand(static_cast<Command::mouseType>(slist.at(2).toInt()));
            break;
        case Command::Text:
            commands[slist.at(1)]    = Command::createTextCommand(Command::removeQuotes(slist.at(2)));
            break;
        case Command::Exec:
            commands[slist.at(1)]    = Command::createExecCommand(slist.at(2));
            break;
        }
    }
}

QString CommandSet::getCommandsUtterance() {    // Return commands utterances for language model
    QStringList ret;
    QMapIterator<QString, Command *> i(commands);
    while (i.hasNext()) {
        i.next();
        QString cmd(i.key());
        ret << cmd;
    }
    ret.sort();
    return ret.join("\n");
}

QString CommandSet::getCommandsCSV() {          // Return commands utterances and action for CSV
    QStringList ret;
    QMapIterator<QString, Command *> i(commands);
    while (i.hasNext()) {
        i.next();
        QString cmd;
        cmd.append(QString("\"%1\", \"%2\", \"%3\", \"%4\"")
                   .arg(i.key()).arg(i.value()->getTypeName()).arg(i.value()->getText()).arg(name));
        ret << cmd;
    }
    return ret.join("\n");
}

void CommandSet::fillTable(QTableWidget *table, CommandSet::ShortcutFormat format) {    //  Fill GUI table
    int     comCount = table->rowCount();
    QMapIterator<QString, Command *> i(commands);
    while (i.hasNext()) {
        table->setRowCount(comCount+1);
        i.next();
        QTableWidgetItem *newItem;
        newItem = new QTableWidgetItem(i.key());
        table->setItem(comCount, COL_UTTERANCE, newItem);
        newItem = new QTableWidgetItem(i.value()->getTypeName());
        table->setItem(comCount, COL_TYPE, newItem);
        if (i.value()->getType() == Command::Text) {
            newItem = new QTableWidgetItem(Command::addQuotes(i.value()->getText()));
        } else if (i.value()->getType() == Command::Key) {
            if (format == CommandSet::QtShortcut)
                newItem = new QTableWidgetItem(SEQ_TO_NATIVE(i.value()->getQtKey()));
            else
                newItem = new QTableWidgetItem(i.value()->getKeySym());
        } else {
            newItem = new QTableWidgetItem(i.value()->getQtKey());
        }
        table->setItem(comCount, COL_COMMAND, newItem);
        newItem = new QTableWidgetItem(name);
        table->setItem(comCount, COL_SET, newItem);
        comCount++;
    }
}
