/* comprendo.cpp  -- Comprendo class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      comprendo.cpp
 * @author    Bruno ANSELME
 * @brief     Comprendo class is the application's engine
 * @details   Load several commands sets together (selection from settings).\n
 *              Call voice recognition.\n
 *              Find command from utterance.\n
 *              Manage mouse glide functions.\n
 *              Manage mouse zoom functions.\n
 *              Performs Key, Text, Mouse and Exec commands types (Comprendo::doCommands()).\n
 *              Meta-commands send signals (will be connected to MainWindow::execMeta).\n
 */

#include <QMessageBox>
#include <QCommandLineParser>
#include "misc.h"
#include "comprendo.h"
#include "guicontrol.h"
#include "selectset.h"
#include "zoombutton.h"

//----------------------------
Comprendo::Comprendo(t_params *p, QObject *parent) : QObject(parent),
    cmdSets(),
    lastCommand()
{
    params      = p;
    guiInit();
    timerGlide  = 0;
    config      = nullptr;
    zooming     = false;
    direction   = None;
    screen = getDesktopRect();
    readSets();
    if (!params->pocketTrace) err_set_logfp(NULL);            // Turn off pocketsphinx output
    connect(this,   SIGNAL(stopGlide()),      this, SLOT(stopGliding()));
}

Comprendo::~Comprendo() {
    closePSphinx();
    guiClose();
}

void Comprendo::readSets() {
    cmdSets.clear();
    QStringList  cmds = SelectSet::getSetFiles(params->langShort);
    QListIterator<QString> cmd(cmds);
    while (cmd.hasNext()) {
        cmdSets.append(new CommandSet(QString("%1/%2.cmd").arg(params->langPath.absolutePath()).arg(cmd.next())));
    }
}

void Comprendo::loadCommandSets() {
    readSets();
    QListIterator<CommandSet *> cmds(cmdSets);
    while (cmds.hasNext()) {
        cmds.next()->loadCommands();
    }
}

QString Comprendo::getCommandsList() {          // Return commands utterances with sets name for reminder
    QStringList ret;
    ret.append(QString("-- %1").arg(tr("Repeated utterance")));
    ret.append(lastCommand);
    QListIterator<CommandSet *> cmds(cmdSets);
    while (cmds.hasNext()) {
        CommandSet *set = cmds.next();
        ret.append(QString("-- %1").arg(set->getName()));
        ret.append(set->getCommandsUtterance());
    }
    return ret.join("\n");
}

QString Comprendo::getCommandsUtterance() {     // Return commands utterances for .lm
    QStringList ret;
    QListIterator<CommandSet *> cmds(cmdSets);
    while (cmds.hasNext()) {
        ret.append(cmds.next()->getCommandsUtterance());
    }
    ret.sort();
    return ret.join("\n");
}

QString Comprendo::getCommandsCSV() {           // Return commands for CSV
    QStringList ret;
    QListIterator<CommandSet *> cmds(cmdSets);
    while (cmds.hasNext()) {
        ret.append(cmds.next()->getCommandsCSV());
    }
    ret.sort();
    return ret.join("\n");
}

void Comprendo::fillTable(QTableWidget *table, CommandSet::ShortcutFormat format) {     // Fill GUI table
    QListIterator<CommandSet *> cmds(cmdSets);
    while (cmds.hasNext()) {
        cmds.next()->fillTable(table, format);
    }
}

QString Comprendo::repeatLastCommand(bool suspended) {
    QString com = lastCommand;
    if (!com.isEmpty())
        doCommand(com, suspended);
    return com;
}

void Comprendo::stopGliding() {
    killTimer(timerGlide);
    timerGlide  = 0;
    direction   = None;
}

void Comprendo::timerEvent(QTimerEvent *t){
    if (t->timerId()==timerGlide) {     // Timer for mouse glide
        int x, y, screenNum;
        switch (direction) {
        case Left:      guiMouseMove(-params->glideStep,  0);                   break;
        case Right:     guiMouseMove( params->glideStep,  0);                   break;
        case Up:        guiMouseMove( 0, -params->glideStep);                   break;
        case Down:      guiMouseMove( 0,  params->glideStep);                   break;
        case UpLeft:    guiMouseMove(-params->glideStep,  -params->glideStep);  break;
        case UpRight:   guiMouseMove( params->glideStep,  -params->glideStep);  break;
        case DownLeft:  guiMouseMove(-params->glideStep,   params->glideStep);  break;
        case DownRight: guiMouseMove( params->glideStep,   params->glideStep);  break;
        case None:                                                              break;
        }
        guiGetMousePos(&x, &y, &screenNum);
        if ((x <= screen.left()) || (x >= screen.right()) || (y <= screen.top()) || (y >= screen.bottom()))
            stopGliding();
    }
}

Command *Comprendo::getCommand(QString cmd) {
    QListIterator<CommandSet *> cmds(cmdSets);
    while (cmds.hasNext()) {
        const QMap<QString, Command *> *commands = cmds.next()->getCommands();
        if (commands->contains(cmd)) {
            return commands->value(cmd);
        }
    }
    return nullptr;
}

#define MOUSEREPEAT(WHAT) for (int j = 0; j < count; j++) { WHAT; } line.append(" : ").append(com->getMouseName()); if (count>1) line.append(QString("(%1)").arg(count));
#define MOUSEGLIDE(WHERE) direction = WHERE; timerGlide  = startTimer(params->glideRate); line.append(" : ").append(com->getMouseName());

bool Comprendo::doCommand(QString &cmd, bool suspended) {
    QString         memo = cmd;
    QStringList     pieces;
    int             count = 1;
    QRegExp         re("(.*) (\\d*)");  // a digit (\d), zero or more times (*)
    QString         line = cmd.leftJustified(20,' ');
    if (re.exactMatch(cmd)) {
        pieces  = re.capturedTexts();
        cmd     = pieces[1];
        count   = pieces[2].toInt();
    }
    bool    done(false);
    QListIterator<CommandSet *> cmds(cmdSets);
    while (cmds.hasNext()) {
        const QMap<QString, Command *> *commands = cmds.next()->getCommands();
        if (commands->contains(cmd)) {
            Command        *com = commands->value(cmd);
            switch (com->getType()) {
            case Command::Meta :            // Meta commands
            {
                Command::metaType typ = com->getMetaType();
                if ((suspended) && (typ != Command::Resume)) break;
                if (typ != Command::Repeat)
                    count = 1;
                line.append(" : ").append(com->getMetaName());
                if (count>1) line.append(QString("(%1)").arg(count));
                for (int j = 0; j < count; j++) {
                    emit metaCommand(typ);
                }
                if (typ == Command::Repeat)
                    line.append(" \"").append(lastCommand).append("\"");
                done = true;
            }
                break;
            case Command::Key :             // Keyboard shortcuts, and shorcuts sequences
                if (suspended) {
                    line.clear();
                } else {
                    lastCommand = memo;
                    line.append(" : ").append(SEQ_TO_NATIVE(com->getQtKey()).toStdString().c_str());
                    if (count>1) line.append(QString("(%1)").arg(count));
                    for (int j = 0; j < count; j++) {
                        QStringList coms = com->getKeySym().split(" ");
                        for (int n = 0; n < coms.count(); n++) {
                            guiSendSequence(coms[n].toStdString().c_str());
                        }
                    }
                }
                done = true;
                break;
            case Command::Mouse :           // Mouse commands
                if (suspended) {
                    line.clear();
                } else {
                    lastCommand = memo;
                    Command::mouseType mous = com->getMouseType();
                    switch (mous) {
                    //  Mouse click commands
                    case Command::Left:
                    case Command::Middle:
                    case Command::Right:
                    case Command::WheelUp:
                    case Command::WheelDown:    MOUSEREPEAT(guiSendClick(static_cast<int>(mous) + 1))                       break;
                    //  Mouse down commands
                    case Command::DownLeft:
                    case Command::DownMiddle:
                    case Command::DownRight:    MOUSEREPEAT(guiSendDown(static_cast<int>(mous - Command::DownLeft) + 1))    break;
                    //  Mouse up commands
                    case Command::UpLeft:
                    case Command::UpMiddle:
                    case Command::UpRight:      MOUSEREPEAT(guiSendUp(static_cast<int>(mous - Command::UpLeft) + 1))        break;
                    // Mouse moves commands
                    case Command::MoveLeft:     MOUSEREPEAT(guiMouseMove(-params->mouseStep, 0))                            break;
                    case Command::MoveRight:    MOUSEREPEAT(guiMouseMove(params->mouseStep, 0))                             break;
                    case Command::MoveUp:       MOUSEREPEAT(guiMouseMove(0, -params->mouseStep))                            break;
                    case Command::MoveDown:     MOUSEREPEAT(guiMouseMove(0, params->mouseStep))                             break;
                    case Command::MoveTo:
                    {
                        line.append(" : ").append(com->getMouseName());
                        zooming = true;
                        ZoomButton w(params, this);
                        w.exec();
                        if (w.hasClicked()) {
                            guiMouseTo(w.where().x(), w.where().y());
                            line.append(QString(" %1,%2").arg(w.where().x()).arg(w.where().y()));
                        }
                        zooming = false;
                    }
                        break;
                    // Mouse glide commands
                    case Command::GlideLeft:        MOUSEGLIDE(Left)        break;
                    case Command::GlideRight:       MOUSEGLIDE(Right)       break;
                    case Command::GlideUp:          MOUSEGLIDE(Up)          break;
                    case Command::GlideDown:        MOUSEGLIDE(Down)        break;
                    case Command::GlideUpLeft:      MOUSEGLIDE(UpLeft)      break;
                    case Command::GlideUpRight:     MOUSEGLIDE(UpRight)     break;
                    case Command::GlideDownLeft:    MOUSEGLIDE(DownLeft)    break;
                    case Command::GlideDownRight:   MOUSEGLIDE(DownRight)   break;
                    }
                }
                done = true;
                break;
            case Command::Text :            // Text input command
                if (suspended) {
                    line.clear();
                } else {
                    lastCommand = memo;
                    line.append(" : \"").append(com->getText().toStdString().c_str()).append("\"");
                    if (count>1) line.append(QString("(%1)").arg(count));
                    for (int j = 0; j < count; j++) {
                        QString txt = com->getText();
                        txt.replace(QRegExp("\\\\n"), "\n");
                        txt.replace(QRegExp("\\\\t"), "\t");
                        guiSendText(txt.toStdString().c_str());
                    }
                }
                done = true;
                break;
            case Command::Exec :            // Exec command
                if (suspended) {
                    line.clear();
                } else {
                    lastCommand = memo;
                    line.append(" : ").append(com->getExec().toStdString().c_str());
                    if (count>1) line.append(QString("(%1)").arg(count));
                    for (int j = 0; j < count; j++) {
                        int pos = com->getExec().indexOf(" ");
                        const QString exe  = com->getExec().left(pos);
                        QString args;
                        if (pos > 0)
                            args    = com->getExec().right(com->getExec().count() - pos);
                        const QStringList argslist = parseCombinedArgString(args);
                        QProcess::startDetached(exe, argslist);
                    }
                }
                done = true;
                break;
            }
        }
        if (done) {
            cmd = line;
            break;
        }
    }
    return done;
}

#undef MOUSEREPEAT
#undef MOUSEGLIDE

bool Comprendo::initPSphinx() {         // Init pocketsphinx
    if (config) return true;            // PocketSphinx already initialized
    QFile lmf(params->lmFile);
    if (lmf.size() == 0) {
        QMessageBox msgBox;
        msgBox.setText(tr("Empty LM file. Check your vocabulary and commands."));
        msgBox.exec();
        return false;
    }
    config = cmd_ln_init(nullptr, ps_args(),
                         TRUE,
                         "-hmm",    params->pocketPath.absolutePath().toStdString().c_str(),
                         "-dict",   params->dictFile.toStdString().c_str(),
                         "-lm",     params->lmFile.toStdString().c_str(),
                         nullptr);
    if (config == nullptr) {
            fprintf(stderr, "Failed to create config object, see log for details\n");
            exit(-1);
    }
    ps = ps_init(config);
    if (ps == nullptr) {
            fprintf(stderr, "Failed to create recognizer, see log for details\n");
            exit(-1);
    }
    if ((ad = ad_open_dev(nullptr, static_cast<int>(static_cast<float32>(cmd_ln_float_r(config,"-samprate"))))) == nullptr)
            E_FATAL("Failed to open audio device\n");
    if (ad_start_rec(ad) < 0)
            E_FATAL("Failed to start recording\n");

    if (ps_start_utt(ps) < 0)
            E_FATAL("Failed to start utterance\n");
    utt_started = FALSE;
    emit statusListening(false);
    return true;
}

void Comprendo::closePSphinx() {
    if (config) {
        ad_close(ad);
        ps_free(ps);
        cmd_ln_free_r(config);
        config = nullptr;
    }
}

QString *Comprendo::recognize_from_microphone() {
    QString *str = new QString();               // Pointer must be deleted by the consumer
    if ((k = ad_read(ad, adbuf, 4096)) < 0)
        E_FATAL("Failed to read audio\n");
    ps_process_raw(ps,
                   adbuf,
                   static_cast<size_t>(k),
                   FALSE, FALSE);
    in_speech = ps_get_in_speech(ps);
    if (in_speech && !utt_started) {
        emit statusListening(true);
        utt_started = TRUE;
        if (timerGlide)
            emit stopGlide();
    }
    hyp = nullptr;
    if (!in_speech && utt_started) {
        emit statusListening(false);
        /* speech -> silence transition, time to start new utterance  */
        ps_end_utt(ps);
        hyp = ps_get_hyp(ps, nullptr );
        if (hyp != nullptr) {
            str->append(hyp);
        }
        if (ps_start_utt(ps) < 0)
            E_FATAL("Failed to start utterance\n");
        utt_started = FALSE;
    }
    return str;
}

// Build wordFile and corpusFile from dictFile and mouseFile
void Comprendo::updateCorpus() {
    // Read dictionary
    QString dict = readFile(params->dictFile);
    // Add words from dictionary
    QStringList words;
    QStringList corpus;
    QStringList wlines(dict.split("\n"));
    QRegExp re(".*\\(\\d+\\).*");
    QRegExp rnum("\\d+");
    for (int i = 0; i < wlines.count(); i++) {
        if (!wlines[i].isEmpty())
            if (!re.exactMatch(wlines[i])) {
                QString     txt = wlines[i].split(" ").at(0);   // Get first word of each line
                if (rnum.exactMatch(txt))                       // if this word is a number
                    corpus << txt;                              //  add to corpus
                words << txt;                                   // always add to words
            }
    }
    writeFile(params->wordFile, words.join(QChar::LineFeed));    // words.txt to build utterances
    // Add words from mouseFile to corpus
    corpus.append(readFile(params->mouseFile).trimmed().split("\n"));
    // Add utterances found in sets selection
    QString     utterances = getCommandsUtterance();
    QStringList lines   = utterances.split("\n");
    for (int i = 0; i < lines.count(); i++) {
        if (!corpus.contains(lines[i]))
            corpus << lines[i];
    }
    corpus.removeDuplicates();
    corpus.sort();
    writeFile(params->corpusFile, corpus.join(QChar::LineFeed).append("\n<s>\n</s>\n"));               // corpus.txt to build language model
}
