/* comprendo.h  -- Comprendo class definition for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COMPRENDO_H
#define COMPRENDO_H

#include <QObject>
#include <QString>
#include <QMap>
#include <QDir>
#include <QProcess>

#include <sphinxbase/err.h>
#include <sphinxbase/ad.h>
#include <pocketsphinx.h>

#include "commandset.h"
#include "params.h"

class Comprendo : public QObject {
    Q_OBJECT
public:
    enum        ShortcutFormat { QtShortcut, XdoShortcut };
    enum        Direction {None, Left, Right, Up, Down, UpLeft, UpRight, DownLeft, DownRight };
    explicit    Comprendo(t_params *p, QObject *parent = nullptr);                  //!< Create object from parameters
    ~Comprendo();

    bool        initPSphinx();                                                      //!< Initialize pocketsphinx
    void        closePSphinx();                                                     //!< Close pocketsphinx
    QString    *recognize_from_microphone();                                        //!< Call pocketsphinx, wait for an utterance completed
    bool        isInitialized()                 { return (config != nullptr); }     //!< Return object state
    Command    *getCommand(QString cmd);                                            //!< Return Command object from utterance
    bool        doCommand(QString &cmd, bool suspended);                            //!< Run Command from utterance. Signal sent for metas
    QString     getCommandsList();                                                  //!< Return formatted commands list for Reminder
    QString     getCommandsUtterance();                                             //!< Return commands list for language model
    QString     getCommandsCSV();                                                   //!< Return formatted commands list for CSV
    QString     getLastCommand()                { return lastCommand; }             //!< Return last command string
    QString     repeatLastCommand(bool suspended);                                  //!< Repeat last command and return last command string
    void        fillTable(QTableWidget *table, CommandSet::ShortcutFormat format);  //!< Fill GUI table with commands names and actions. Calls CommandSet::fillTable
    void        readSets();                                                         //!< Read selected commands sets from settings
    bool        isZooming()                     { return zooming; }                 //!< True when mouse zooming in progress
    bool        isGliding()                     { return (direction != None); }     //!< True when mouse gliding in progress
    void        updateCorpus();                                                     //!< Update corpusFile before LM generation
signals:
    void        stopGlide();                                                        //!< Mouse glide asked to stopped
    void        statusListening(bool);                                              //!< Listening status has changed
    void        metaCommand(Command::metaType);                                     //!< Signals a meta command is required (done in MainWindow::execMeta)

public slots:
    void        loadCommandSets();                                                  //!< Reload commands sets from settings
protected:
    void        timerEvent(QTimerEvent *event);                                     //!< Handle mouse glide timer
protected slots:
    void        stopGliding();                                                      //!< Mouse glide was stopped
private:
    t_params               *params;
    QList<CommandSet *>     cmdSets;
    QString                 lastCommand;
    QRect                   screen;
    // Mouse move data
    bool                    zooming;            // Zooming mode flag
    Direction               direction;          // Mouse glide direction
    int                     timerGlide;         // Mouse is gliding when != 0
    // Pocketsphinx data and buffer
    cmd_ln_t               *config;
    ps_decoder_t           *ps;
    ad_rec_t               *ad;
    int16                   adbuf[4096];
    uint8                   utt_started, in_speech;
    int32                   k;
    char const             *hyp;
};

#endif // COMPRENDO_H
