/* editcommand.cpp  -- EditCommand class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      editcommand.cpp
 * @author    Bruno ANSELME
 * @brief     EditCommand class definition
 * @details   Command editor.\n
 *              Handle all commands types in a dialog.\n
 */

#include <QFileDialog>
#include <QMetaMethod>
#include <QSettings>
#include "app_version.h"
#include "editcommand.h"

EditCommand::EditCommand(QString utt, Command &cmd) : QDialog(),
    ui(new Ui::dialCommand)
{
    lastType = Command::Meta;
    init();

    ui->leUtterance->setText(utt);
    switch (cmd.getType()) {
    case Command::Meta:
        ui->btMeta->click();
        ui->cbMeta->setCurrentIndex(static_cast<int>(cmd.getMetaType()));
        break;
    case Command::Key:
        ui->btKey->click();
        ui->leShortcut->setText(SEQ_TO_NATIVE(cmd.getQtKey()));
        break;
    case Command::Mouse:
        ui->btMouse->click();
        ui->cbMouse->setCurrentIndex(static_cast<int>(cmd.getMouseType()));
        break;
    case Command::Text:
        ui->btText->click();
        ui->leTextInput->setText(cmd.getText());
        break;
    case Command::Exec:
        ui->btExec->click();
        ui->leProgram->setText(cmd.getText());
        break;
    }
    QSettings settings(APP_ORG, APP_NAME);
    move(settings.value("comPos", QPoint(200, 300)).toPoint());
    resize(settings.value("comSize", QSize(467, 283)).toSize());
}

EditCommand::~EditCommand() {
    QSettings settings(APP_ORG, APP_NAME);
    settings.setValue("comPos",            pos());
    settings.setValue("comSize",           size());
}

void EditCommand::init() {
    ui->setupUi(this);
    buts.addButton(ui->btMeta);
    buts.addButton(ui->btKey);
    buts.addButton(ui->btMouse);
    buts.addButton(ui->btText);
    buts.addButton(ui->btExec);
    buts.setId(ui->btMeta, 0);
    buts.setId(ui->btKey, 1);
    buts.setId(ui->btMouse, 2);
    buts.setId(ui->btText, 3);
    buts.setId(ui->btExec, 4);
    connect(ui->btMeta,        SIGNAL(clicked()),          this,   SLOT(toMeta()));
    connect(ui->btKey,         SIGNAL(clicked()),          this,   SLOT(toKey()));
    connect(ui->btMouse,       SIGNAL(clicked()),          this,   SLOT(toMouse()));
    connect(ui->btText,        SIGNAL(clicked()),          this,   SLOT(toText()));
    connect(ui->btExec,        SIGNAL(clicked()),          this,   SLOT(toExec()));
    connect(ui->kseqEdit,      SIGNAL(editingFinished()),  this,   SLOT(newKeySequence()));
    ui->cbMeta->addItems(Command::getMetaNames());
    ui->cbMouse->addItems(Command::getMouseNames());
}

void EditCommand::clearSignals(Command::comType newtype) {
    switch (lastType) {
    case Command::Key:
        disconnect(ui->leShortcut, SIGNAL(textChanged(const QString &)), this, SLOT(checkSequence(const QString &)));
        break;
    case Command::Exec:
        disconnect(ui->leProgram,  SIGNAL(textChanged(const QString &)), this, SLOT(checkExec(const QString &)));
        break;
    default:
        break;
    }
    lastType = newtype;
}

void EditCommand::toMeta() {
    clearSignals(Command::Meta);
    ui->gbMeta->setEnabled(true);
    ui->gbKey->setEnabled(false);
    ui->gbMouse->setEnabled(false);
    ui->gbText->setEnabled(false);
    ui->gbExec->setEnabled(false);
    ui->stack->setCurrentIndex(0);
    ui->btOk->setEnabled(true);
}

void EditCommand::toKey() {
    clearSignals(Command::Key);
    ui->gbMeta->setEnabled(false);
    ui->gbKey->setEnabled(true);
    ui->gbMouse->setEnabled(false);
    ui->gbText->setEnabled(false);
    ui->gbExec->setEnabled(false);
    ui->stack->setCurrentIndex(1);
    connect(ui->leShortcut,    SIGNAL(textChanged(const QString &)), this, SLOT(checkSequence(const QString &)));
    checkSequence(ui->leShortcut->text());
}

void EditCommand::toMouse() {
    clearSignals(Command::Mouse);
    ui->gbMeta->setEnabled(false);
    ui->gbKey->setEnabled(false);
    ui->gbMouse->setEnabled(true);
    ui->gbText->setEnabled(false);
    ui->gbExec->setEnabled(false);
    ui->stack->setCurrentIndex(2);
    ui->btOk->setEnabled(true);
}

void EditCommand::toText() {
    clearSignals(Command::Text);
    ui->gbMeta->setEnabled(false);
    ui->gbKey->setEnabled(false);
    ui->gbMouse->setEnabled(false);
    ui->gbText->setEnabled(true);
    ui->gbExec->setEnabled(false);
    ui->stack->setCurrentIndex(3);
    ui->btOk->setEnabled(true);
}

void EditCommand::toExec() {
    clearSignals(Command::Exec);
    ui->gbMeta->setEnabled(false);
    ui->gbKey->setEnabled(false);
    ui->gbMouse->setEnabled(false);
    ui->gbText->setEnabled(false);
    ui->gbExec->setEnabled(true);
    ui->stack->setCurrentIndex(4);
    connect(ui->leProgram,    SIGNAL(textChanged(const QString &)), this, SLOT(checkExec(const QString &)));
    checkExec(ui->leProgram->text());
}

Command::comType    EditCommand::getType()  { return static_cast<Command::comType>(buts.checkedId());               }
Command::metaType   EditCommand::getMeta()  { return static_cast<Command::metaType>(ui->cbMeta->currentIndex());    }
Command::mouseType  EditCommand::getMouse() { return static_cast<Command::mouseType>(ui->cbMouse->currentIndex());  }

QString         EditCommand::getText()      { return ui->leTextInput->text();   }
QString         EditCommand::getExec()      { return ui->leProgram->text();     }
QKeySequence    EditCommand::getSequence()  { QKeySequence  seq(ui->leShortcut->text()); return seq;    }

void EditCommand::newKeySequence() {
    ui->leShortcut->setText(ui->kseqEdit->keySequence().toString(QKeySequence::NativeText));
    ui->kseqEdit->setKeySequence(QKeySequence(""));
    checkSequence(ui->leShortcut->text());
}

void EditCommand::checkSequence(const QString &s) {
    QKeySequence  seq(s);
    if ((!s.isEmpty()) && (s == seq.toString(QKeySequence::NativeText))) {
        ui->lbKeyStatus->setText(tr("Valid shortcut"));
        ui->btOk->setEnabled(true);
    } else {
        ui->lbKeyStatus->setText(tr("Invalid shortcut"));
        ui->btOk->setEnabled(false);
    }
}

void EditCommand::checkExec(const QString &s) {
    QStringList parts = s.split(QRegExp("\\s+"));
    QFileInfo info(parts.first());
    if ((!info.isDir()) && info.isExecutable()) {
        ui->lbExecStatus->setText(tr("<b>%1</b> is an executable file").arg(parts.first()));
        ui->btOk->setEnabled(true);
    } else {
        ui->lbExecStatus->setText(tr("<b>%1</b> is not an executable file").arg(parts.first()));
        ui->btOk->setEnabled(false);
    }
}

