/* editset.cpp  -- EditSet class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      editset.cpp
 * @author    Bruno ANSELME
 * @brief     EditSet class definition
 * @details   Commands set editor.\n
 *              Build utterances with consecutive clicks.\n
 *              Add/edit/remove commands.\n
 */

#include <QFile>
#include <QKeyEvent>
#include <QEventLoop>
#include <QSettings>
#include "app_version.h"
#include "misc.h"
#include "comprendo.h"
#include "editset.h"
#include "editcommand.h"

#define CL_TYPE        0
#define CL_UTTERANCE   1
#define CL_COMMAND     2

EditSet::EditSet(t_params &p, QString setname) : QDialog(),
    ui(new Ui::dialCommandSet),
    recording(false)
{
    params  = p;
    ui->setupUi(this);
    ui->lbCmdName->setText(setname);
    cmdfile    = QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(setname);
    ui->btAdd->setEnabled(false);
    ui->btRemove->setEnabled(false);
    ui->btChange->setEnabled(false);
    ui->btModify->setEnabled(false);
// Read vocabulary
    loadVocabularyList(params.wordFile);
    comCount = 0;
    newcom   = false;
    ui->tabCom->setRowCount(comCount);
    ui->tabCom->verticalHeader()->setSectionResizeMode(QHeaderView::Fixed);
    ui->tabCom->verticalHeader()->setDefaultSectionSize(18);

// Read commands
    loadCommandTable();
    connect(ui->btAdd,         SIGNAL(clicked()),                   this,   SLOT(addCommand()));
    connect(ui->btRemove,      SIGNAL(clicked()),                   this,   SLOT(removeCommand()));
    connect(ui->btChange,      SIGNAL(clicked()),                   this,   SLOT(changeUtterance()));
    connect(ui->tabCom,        SIGNAL(cellDoubleClicked(int,int)),  this,   SLOT(editionCommand(int,int)));
    connect(ui->btModify,      SIGNAL(clicked()),                   this,   SLOT(editionCommand()));
    connect(ui->cbTooltip,     SIGNAL(toggled(bool)),               this,   SLOT(disableTooltips(bool)));
    connect(ui->lstVoc,        SIGNAL(itemSelectionChanged()),      this,   SLOT(updateUtterance()));
    connect(ui->tabCom,        SIGNAL(currentCellChanged(int,int,int,int)),
                                                                    this,   SLOT(tableActivated(int, int)));
    disableTooltips(false);
    QSettings settings(APP_ORG, APP_NAME);
    move(settings.value("setPos", QPoint(350, 150)).toPoint());
    resize(settings.value("setSize", QSize(860, 720)).toSize());
    ui->splitter->restoreState(settings.value("setSplit").toByteArray());
}

EditSet::~EditSet() {
    QSettings settings(APP_ORG, APP_NAME);
    settings.setValue("setPos",            pos());
    settings.setValue("setSize",           size());
    settings.setValue("setSplit",          ui->splitter->saveState());
}

void EditSet::tableActivated(int, int) {
    ui->lstVoc->clearSelection();
    ui->btAdd->setEnabled(false);
    ui->btRemove->setEnabled(true);
    ui->btChange->setEnabled(true);
    ui->btModify->setEnabled(true);
}

void EditSet::changeUtterance() {
    QTableWidgetItem *item = ui->tabCom->item(ui->tabCom->currentRow(), CL_UTTERANCE);
    item->setText(ui->lbUtterance->text().trimmed());
    item->setBackground(QBrush(Qt::white));
    ui->btAdd->setEnabled(false);
}

void EditSet::updateUtterance() {   // Update lbUtterance field in UI
    QList<QListWidgetItem *> selection = ui->lstVoc->selectedItems();
    QString cmd;
    for (int i = 0; i < selection.count(); i++) {
        cmd.append(selection[i]->text()).append(" ");
    }
    ui->lbUtterance->setText(cmd);
    ui->btRemove->setEnabled(false);
    ui->btAdd->setEnabled(!cmd.isEmpty());
}

void EditSet::loadVocabularyList(QString name) {
    QString txt = readFile(name);
    vocabulary = txt.split("\n");
    for (int i = 0; i < vocabulary.count(); i++) {
        if (!vocabulary[i].contains("("))
            ui->lstVoc->addItem(vocabulary[i]);
    }
}

void EditSet::addField(int col, QString s, bool ok) {
    QTableWidgetItem *newItem = new QTableWidgetItem(s);
    newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    if (!ok)
        newItem->setBackground(QBrush(Qt::lightGray));
    ui->tabCom->setItem(comCount, col, newItem);
}

void EditSet::loadCommandTable() {
    QFile file(cmdfile);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;
    QTextStream f(&file);
    QString     cmd;
    QStringList slist;

    while (f.readLineInto(&cmd)) {   // Read command utterance
        ui->tabCom->setRowCount(comCount+1);
        slist = cmd.split(QString(" : "));

        // Check if utterance's words are all in vocabulary list
        bool ok = true;
        QString     utt   = slist.at(1);
        QStringList words = utt.split(" ");
        for (int i = 0; i < words.count(); i++) {
            if (!vocabulary.contains(words[i]))
                ok = false;
        }

        addField(CL_TYPE, Command::getTypeName(static_cast<Command::comType>(slist.at(0).toInt())), ok);
        addField(CL_UTTERANCE, utt, ok);
        switch (static_cast<Command::comType>(slist.at(0).toInt())) {
        case Command::Key:
            addField(CL_COMMAND, SEQ_TO_NATIVE(slist.at(2)), ok);
            break;
        case Command::Meta:
            addField(CL_COMMAND, Command::getMetaName(static_cast<Command::metaType>(slist.at(2).toInt())), ok);
            break;
        case Command::Mouse:
            addField(CL_COMMAND, Command::getMouseName(static_cast<Command::mouseType>( slist.at(2).toInt())), ok);
            break;
        case Command::Text:
            addField(CL_COMMAND, Command::addQuotes(slist.at(2)), ok);
            break;
        case Command::Exec:
            addField(CL_COMMAND, slist.at(2), ok);
            break;
        }
        comCount++;
    }
    ui->tabCom->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

void EditSet::removeCommand() {
    if (ui->tabCom->currentRow() != -1) {
        ui->tabCom->removeRow(ui->tabCom->currentRow());
        comCount--;
        ui->tabCom->resizeColumnsToContents();
    }
}

void EditSet::addCommand() {
    QList<QListWidgetItem *> selection = ui->lstVoc->selectedItems();
    if (selection.isEmpty()) return;
    ui->tabCom->setSortingEnabled(false);
    QString cmd;
    for (int i = 0; i < selection.count(); i++) {
                cmd.append(selection[i]->text()).append(" ");
    }
    ui->tabCom->setRowCount(comCount+1);
    addField(CL_TYPE, Command::getTypeName(Command::Key));
    addField(CL_UTTERANCE, cmd.trimmed());
    addField(CL_COMMAND, "");
    ui->tabCom->setCurrentCell(comCount,0);
    ui->tabCom->scrollToBottom();
    comCount++;
    ui->tabCom->selectRow(comCount-1);
    newcom = true;
    ui->tabCom->setSortingEnabled(true);
    emit ui->tabCom->cellDoubleClicked(ui->tabCom->currentRow(),0);
}

void EditSet::editionCommand() {
    editionCommand(ui->tabCom->currentRow(),0);
}

void EditSet::editionCommand(int row, int) {
    Command *cmd = nullptr;
    switch (Command::typeFromName(ui->tabCom->item(row, CL_TYPE)->text())) {
    case Command::Meta:
        cmd = Command::createMetaCommand(Command::metaFromName(ui->tabCom->item(row, CL_COMMAND)->text()));
        break;
    case Command::Key:
        cmd = Command::createKeyCommand(SEQ_TO_PORTABLE(ui->tabCom->item(row, CL_COMMAND)->text()));
        break;
    case Command::Mouse:
        cmd = Command::createMouseCommand(Command::mouseFromName(ui->tabCom->item(row, CL_COMMAND)->text()));
        break;
    case Command::Text:
        cmd = Command::createTextCommand(ui->tabCom->item(row, CL_COMMAND)->text());
        break;
    case Command::Exec:
        cmd = Command::createExecCommand(ui->tabCom->item(row, CL_COMMAND)->text());
        break;
    }
    EditCommand dial(ui->tabCom->item(row, CL_UTTERANCE)->text(), *cmd);
    int dialogCode = dial.exec();
    if (dialogCode == QDialog::Accepted) {
        Command::comType ctyp = dial.getType();
        ui->tabCom->item(row, CL_TYPE)->setText(Command::getTypeName(ctyp));
        switch (ctyp) {
        case Command::Meta:
            ui->tabCom->item(row, CL_COMMAND)->setText(Command::getMetaName(dial.getMeta()));
            break;
        case Command::Key:
            ui->tabCom->item(row, CL_COMMAND)->setText(SEQ_TO_NATIVE(dial.getSequence().toString()));
            break;
        case Command::Mouse:
            ui->tabCom->item(row, CL_COMMAND)->setText(Command::getMouseName(dial.getMouse()));
            break;
        case Command::Text: {
            QString cmdtxt = Command::addQuotes(dial.getText());
            ui->tabCom->item(row, CL_COMMAND)->setText(cmdtxt);
        }
            break;
        case Command::Exec:
            ui->tabCom->item(row, CL_COMMAND)->setText(dial.getExec());
            break;
        }
        ui->tabCom->resizeColumnsToContents();
    }
    if (dialogCode == QDialog::Rejected) {
        if (newcom) {
            removeCommand();
        }
    }
    newcom = false;
    if (cmd) delete cmd;
}

void EditSet::disableTooltips(bool b) {
    if (b) {
        ui->tabCom->setToolTip("");
        ui->lstVoc->setToolTip("");
    }  else {
        ui->tabCom->setToolTip(tr("Double click to edit a command"));
        ui->tabCom->setToolTipDuration(3000);
        ui->lstVoc->setToolTip(tr("Build an utterance with multiple words with one click followed by Ctrl+click"));
        ui->lstVoc->setToolTipDuration(3000);
    }
}

void EditSet::accept() {
    QFile file(cmdfile);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return;
    QTextStream f(&file);
    for (int i = 0; i < ui->tabCom->rowCount(); i++) {
        Command::comType ctyp = Command::typeFromName(ui->tabCom->item(i, CL_TYPE)->text());
        f <<            ctyp;                                                                   // Save command type
        f << " : " <<   ui->tabCom->item(i, CL_UTTERANCE)->text();                              // Save command utterance
        switch (ctyp) {
        case Command::Meta:
            f << " : " <<  Command::metaFromName(ui->tabCom->item(i, CL_COMMAND)->text());      // Save meta command
            break;
        case Command::Key:
            f << " : " <<   SEQ_TO_PORTABLE(ui->tabCom->item(i, CL_COMMAND)->text());           // Save command shortcut
            break;
        case Command::Mouse:
            f << " : " <<  Command::mouseFromName(ui->tabCom->item(i, CL_COMMAND)->text());     // Save mouse command
            break;
        case Command::Text:
            f << " : " <<  ui->tabCom->item(i, CL_COMMAND)->text() ;                            // Save text input
            break;
        case Command::Exec:
            f << " : " <<  ui->tabCom->item(i, CL_COMMAND)->text();                             // Save program or script to start
            break;
        }
        f << "\n";
    }
    file.close();
    emit setUpdated();
    QDialog::accept();
}
