/* editvocabulary.cpp  -- EditVocabulary class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      editvocabulary.cpp
 * @author    Bruno ANSELME
 * @brief     EditVocabulary class definition
 * @details   Vocabulary editor.\n
 *              Pick up words from dictionary and them to vocabulary.
 */

#include <QRegExp>
#include <QCollator>
#include <QSettings>
#include <QScrollBar>
#include "app_version.h"
#include "misc.h"
#include "fileviewer.h"
#include "editvocabulary.h"

EditVocabulary::EditVocabulary(t_params &p, Comprendo *comp) : QDialog(),
    ui(new Ui::dialVocabulary),
    lines(),
//    url("http://www.speech.cs.cmu.edu/tools/lmtool-new.html"),    // On line CMU tools version 3
    loading(true)
{
    params  = p;
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->setupUi(this);
    what = comp;

    // Load language dictionary to ListWidget (fr.dict or en.dict)
    QString filename = QString("%1/%2.dict").arg(params.appPath.absolutePath()).arg(params.lang.left(2));
    lines = readFile(filename).split("\n");
    ui->lstLanguage->addItems(lines);
    ui->lbCount->setText(QString("%1").arg(ui->lstLanguage->count()));
    // Load user vocabulary to PlainTextEdit
    ui->textVocabulary->setPlainText(readFile(params.dictFile));

    connect(ui->btFilter,          SIGNAL(clicked()),                      this,   SLOT(filterLanguage()));
    connect(ui->lstLanguage,       SIGNAL(doubleClicked(QModelIndex)),     this,   SLOT(addWord()));
    connect(ui->btSort,            SIGNAL(clicked()),                      this,   SLOT(sortVocabulary()));
    connect(ui->btStats,           SIGNAL(clicked()),                      this,   SLOT(showStats()));
    ui->leFilter->setFocus();
    QSettings settings(APP_ORG, APP_NAME);
    move(settings.value("vocPos", QPoint(250, 250)).toPoint());
    resize(settings.value("vocSize", QSize(920, 640)).toSize());
}

EditVocabulary::~EditVocabulary() {
    QSettings settings(APP_ORG, APP_NAME);
    settings.setValue("vocPos",            pos());
    settings.setValue("vocSize",           size());
}

void EditVocabulary::showStats() {
    QHash<QString, int> wcount;
    QString txt = readFile(params.wordFile);
    QStringList allwords = txt.split("\n");
    QRegExp rdigit("\\d+");   // Regular expression to detect digits
    for (int i=0; i < allwords.count(); i++) {
        if (!rdigit.exactMatch(allwords[i]))        // Exclude digits
            wcount[allwords[i]] = 0;                // Init counter for this word
    }
    QString utts = what->getCommandsUtterance();
    utts.replace(" ","\n");                         // Vocabulary from commands
    utts.append(readFile(params.mouseFile));        // Vocabulary from mouse move to
    QStringList usedwords = utts.split("\n");       // Contains a list of each word used by enabled commands sets
    for (int i=0; i < usedwords.count(); i++) {
        if (!rdigit.exactMatch(usedwords[i]))       // Exclude digits
            wcount[usedwords[i]]++;
    }
    QString out;
    out.append(tr("Words usage by selected command sets and mouse MoveTo\n"));
    out.append(tr("Different words        : %1\n").arg(wcount.count()));
    out.append(tr("Word                   : Usage count\n"));
    out.append(QString("----------------------------------------\n"));
    for (int i=0; i < allwords.count(); i++) {
        if (!rdigit.exactMatch(allwords[i]))        // Exclude digits
            out.append(QString("%1 \t: %2\n").arg(allwords[i],-20).arg(wcount[allwords[i]]));
    }
    FileViewer fv(out);
    fv.setTitle(tr("Statistics"));
    fv.exec();
}

void EditVocabulary::filterLanguage() {
    QRegExp re(ui->leFilter->text(), Qt::CaseInsensitive, QRegExp::Wildcard);
    ui->lstLanguage->clear();
    ui->lstLanguage->addItems(lines.filter(re));
    ui->lbCount->setText(QString("%1").arg(ui->lstLanguage->count()));
    ui->lstLanguage->setCurrentRow(0);
}

void EditVocabulary::sortVocabulary(QString &txt) {
    txt.replace(QRegExp("\\(\\d+\\)"), "");         // Remove parenthesis, will be re-inserted later
    QStringList lines   = txt.split("\n");
    for (int i = 0; i < lines.count(); i++) {       // Trim lines (whitespace removed from the start and the end)
        lines[i] = lines[i].simplified();
    }
    txt.clear();
    QCollator coll;                                 // Natural sort with std::sort and QCollator class...
    coll.setNumericMode(true);
    std::sort(lines.begin(), lines.end(), [&](const QString& s1, const QString& s2){ return coll.compare(s1, s2) < 0; });
    // Delete empty and duplicate lines, append to txt
    QSet<int>           linesSeen;                  // Contains whole lines hash
    QHash<QString, int> wordcount;                  // Contains word count for each word, (build parenthesis count with this value)
    QRegExp             rsplit("(\\S+)(\\s+.*)");   // Regular expression to split word and phonetics
    QRegExp             rskip(".*[\\(\\)].*");      // Regular expression to skip lines with parenthesis
    QStringList         parts;                      // Contains captured segments
    for (int i = 0; i < lines.count(); i++) {       // Delete duplicate lines
        if (lines[i] == "")             continue;   // No empty line
        if (rskip.exactMatch(lines[i])) continue;   // Skip lines containing parenthesis
        int h = static_cast<int>(qHash(lines[i]));
        if(linesSeen.contains(h))   continue;       // Line already seen if h already exists
        linesSeen << h;                             // Add this hash to lineSeen
        if (rsplit.exactMatch(lines[i])) {          // Extract word (parts[1]) and phonetics (parts[2])
            parts  = rsplit.capturedTexts();
            parts[1] = parts[1].toLower();          // lowercase word
            if (wordcount.contains(parts[1])) {
                wordcount[parts[1]]++;              // Increment counter for this word
                parts[1].append(QString("(%1)").arg(wordcount[parts[1]]));      // Append parenthesis around word count
            } else {
                wordcount[parts[1]] = 1;            // Start counter for this word
            }
            txt.append(parts[1]).append(parts[2]).append("\n");
        }
    }
}

void EditVocabulary::sortVocabulary() {
    int     vpos    = ui->textVocabulary->verticalScrollBar()->sliderPosition();    // Save text scroll position
    QString txt     = ui->textVocabulary->toPlainText();
    sortVocabulary(txt);
    ui->textVocabulary->setPlainText(txt);
    ui->textVocabulary->verticalScrollBar()->setSliderPosition(vpos);               // Restore text scroll position
    writeFile(params.dictFile, txt);
}

void EditVocabulary::accept() {
    sortVocabulary();
    QDialog::accept();
}

void EditVocabulary::paintEvent(QPaintEvent *) {
    if (loading) {
        QApplication::restoreOverrideCursor();
        loading = false;
   }
}
void EditVocabulary::addWord() {
    int selected = ui->lstLanguage->currentRow();
    ui->textVocabulary->moveCursor(QTextCursor::Start);
    ui->textVocabulary->insertPlainText(ui->lstLanguage->item(selected)->text());
    ui->textVocabulary->insertPlainText(QString("\n"));
}
