/* guicontrol.cpp  -- Xdo access functions implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
// Interface to xdotool
// For a future Wayland version :
//   - Change xdo functions to appropriate functions.
//   - Check if a translation table is needed.
//   - Check guiKeySequence function for update.
//   - Everything is here.
//-------------------------------------------------

/**
 * @file      guicontrol.cpp
 * @author    Bruno ANSELME
 * @brief     Interface with libxdo
 * @details   Interaction with GUI.\n
 *            Send keys sequences, text, mouse control to libxdo.
 */

#include <QHash>
extern "C" {
#include <xdo.h>
}

static xdo_t   *xdo = nullptr;                  // Pointer to wdo environment
static QHash<QString, QString>   qtToKeysym;    // Hash table Qt keys to keysyms

/**
 * @brief   Initialize static Qt to KeySym translation table
 * @details Data for Qt keys to KeySyms translation is hard coded.\n
 *            Used only by guiKeySequence function below.
 */
void initGuiKeys() {
    if (qtToKeysym.isEmpty()) {     // Run only once
        qtToKeysym[QString("Ctrl")]         = QString("ctrl");
        qtToKeysym[QString("Shift")]        = QString("shift");
        qtToKeysym[QString("Alt")]          = QString("alt");
        qtToKeysym[QString("Meta")]         = QString("super");
        qtToKeysym[QString("PgUp")]         = QString("Prior");
        qtToKeysym[QString("PgDown")]       = QString("Next");
        qtToKeysym[QString("Backspace")]    = QString("BackSpace");
        qtToKeysym[QString("Esc")]          = QString("Escape");
        qtToKeysym[QString("Del")]          = QString("Delete");
        qtToKeysym[QString("Ins")]          = QString("Insert");

        qtToKeysym[QString("Print")]        = QString("Print");
        qtToKeysym[QString("ScrollLock")]   = QString("Scroll_Lock");
        qtToKeysym[QString("NumLock")]      = QString("Num_Lock");
        qtToKeysym[QString("CapsLock")]     = QString("Caps_Lock");
        qtToKeysym[QString("Pause")]        = QString("Pause");

        qtToKeysym[QString("Num++")]        = QString("KP_Add");
        qtToKeysym[QString("Num+-")]        = QString("KP_Subtract");
        qtToKeysym[QString("Num+/")]        = QString("KP_Divide");
        qtToKeysym[QString("Num+.")]        = QString("KP_Delete");

        // Numlocked numpad keys inverted between Qt and libxdo !!!
        qtToKeysym[QString("Num+0")]        = QString("KP_Insert");
        qtToKeysym[QString("Num+1")]        = QString("KP_End");
        qtToKeysym[QString("Num+2")]        = QString("KP_Down");
        qtToKeysym[QString("Num+3")]        = QString("KP_Page_Down");
        qtToKeysym[QString("Num+4")]        = QString("KP_Left");
        qtToKeysym[QString("Num+6")]        = QString("KP_Right");
        qtToKeysym[QString("Num+7")]        = QString("KP_Home");
        qtToKeysym[QString("Num+8")]        = QString("KP_Up");
        qtToKeysym[QString("Num+9")]        = QString("KP_Page_Up");

        qtToKeysym[QString("Num+Ins")]          = QString("KP_0");
        qtToKeysym[QString("Num+End")]          = QString("KP_1");
        qtToKeysym[QString("Num+Down")]         = QString("KP_2");
        qtToKeysym[QString("Num+Page_Down")]    = QString("KP_3");
        qtToKeysym[QString("Num+Left")]         = QString("KP_4");
        qtToKeysym[QString("Num+5")]            = QString("KP_5");
        qtToKeysym[QString("Num+Right")]        = QString("KP_6");
        qtToKeysym[QString("Num+Home")]         = QString("KP_7");
        qtToKeysym[QString("Num+Up")]           = QString("KP_8");
        qtToKeysym[QString("Num+Page_Up")]      = QString("KP_9");

        // Media keys
        qtToKeysym[QString("Volume Mute")]      = QString("XF86AudioMute");
        qtToKeysym[QString("Volume Up")]        = QString("XF86AudioRaiseVolume");
        qtToKeysym[QString("Volume Down")]      = QString("XF86AudioLowerVolume");
        qtToKeysym[QString("Media Previous")]   = QString("XF86AudioPrev");
        qtToKeysym[QString("Media Next")]       = QString("XF86AudioNext");
        qtToKeysym[QString("Media Play")]       = QString("XF86AudioPlay");
        qtToKeysym[QString("Media Stop")]       = QString("XF86AudioStop");
    }
}

// Xdo interface
void guiInit() {
    if (xdo) return;
    initGuiKeys();
    xdo = xdo_new(nullptr);
}
void guiClose() {
    if (xdo) {
        xdo_free(xdo);
        xdo = nullptr;
    }
}
const char *guiVersion() {
    return xdo_version();
}
int guiScreenSize(unsigned int *width, unsigned int *height, int screen) {
    return xdo_get_viewport_dimensions(xdo, width, height, screen);
}
void guiSendSequence(const char *seq) {
    xdo_send_keysequence_window(xdo, CURRENTWINDOW, seq, 1);
    usleep(1200);
}
void guiSendText(const char *string) {
    xdo_enter_text_window(xdo, CURRENTWINDOW, string, 1200);    // delay in microseconds
}
void guiSendClick(int button) {                                 // Left 1, middle 2, right 3, wheel up 4, wheel down 5
    xdo_click_window(xdo, CURRENTWINDOW, button);
}
void guiSendDown(int button) {                                  // Left 1, middle 2, right 3, wheel up 4, wheel down 5
    xdo_mouse_down(xdo, CURRENTWINDOW, button);
}
void guiSendUp(int button) {                                    // Left 1, middle 2, right 3, wheel up 4, wheel down 5
    xdo_mouse_up(xdo, CURRENTWINDOW, button);
}
void guiMouseMove(int x, int y) {                               // Mouse relative moves
    xdo_move_mouse_relative(xdo, x, y);
}
void guiMouseTo(int x, int y) {                                 // Mouse absolute moves
    xdo_move_mouse(xdo, x, y, 0);
}
void guiGetMousePos(int *x, int *y, int *screen) {
    xdo_get_mouse_location(xdo, x, y, screen);
}

// Convert QKeySequence into KeySyms for guiSendSequence
QString guiKeySequence(QString cmd) {
    QStringList keysyms = cmd.split(", ");                      // Split multiple key sequences (ex: "Ctrl+W, Alt+Tab")
    for (int k = 0; k < keysyms.count(); k++) {
        QHashIterator<QString, QString> i(qtToKeysym);
        while (i.hasNext()) {
            i.next();
            keysyms[k].replace(i.key(), i.value());             // Translate all Ctrl, Alt, Meta etc found into KeySyms value
        }
        QRegExp lastletter("(.*\\+)(\\w)$");
        if (lastletter.exactMatch(keysyms[k])) {                // Lowercase last letter after "+" for KeySyms
            QStringList     pieces;
            pieces  = lastletter.capturedTexts();
            keysyms[k] = pieces[1] + pieces[2].toLower();
        }
    }
    cmd = keysyms.join(" ");                                    // Join multiple key sequences
    return cmd;
}
