/* languagemodeler.cpp  -- LanguageModeler class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      languagemodeler.cpp
 * @author    Bruno ANSELME
 * @brief     LanguageModeler class definition
 * @details   Build a language model from the corpusFile and the wordFile.
 */

#include <QDateTime>
#include <QMessageBox>
#include <app_version.h>
#include <math.h>
#include "languagemodeler.h"
#include "fileviewer.h"
#include "misc.h"

LanguageModeler::LanguageModeler(t_params *p) : params(p) {}

// Code below is Qt/C++ translation from quick_lm.pl
// Results are a bit different from original perl script, but work fine.
// ADD_GRAM simulate perl's behavior. Create new key and set to 1, else increment value.
#define ADD_GRAM(map, key) if (map.contains(key)) { map[key]++; } else { map[key] = 1;}
void LanguageModeler::quickLM() {
    double  log_10      = log(10);
    double  discount    = 0.5;
    if (discount < 0.1) discount = 0.1;                                     // Safety checks
    if (discount > 0.9) discount = 0.9;
    double  deflator    = 1.0 - discount;
    QStringList corpusLines = readFile(params->corpusFile).split("\n");     // Read file and split lines
    // Create count tables
    QMap<QString,   int>    unigram;                                        // Initialize empty maps
    QMap<QString,   int>    bigram;
    QMap<QString,   int>    trigram;
    // Fill count tables (unigram, bigram and trigram)
    int         sent_cnt = 0;
    QRegExp wordSplit("\\s");
    for (int i = 0; i < corpusLines.count(); i++) {
        corpusLines[i] = corpusLines[i].trimmed();
        if (corpusLines.isEmpty())  continue;                               // Skip empty lines
        sent_cnt++;
        QStringList words = corpusLines[i].split(wordSplit);                // Split words from line
        int  j;
        for (j = 0; j < words.count(); j++) {
            ADD_GRAM(unigram, words[j]);
            if (j == words.count()-1) {
                if (words[j] != "") {  ADD_GRAM(bigram,  (words[j]+" ")) }
            }
            if (j < words.count()-1) { ADD_GRAM(bigram,  (words[j]+" "+words[j+1]))                 }
            if (j < words.count()-2) { ADD_GRAM(trigram, (words[j]+" "+words[j+1]+" "+ words[j+2])) }
        }
    }
    // Add all vocabulary words if asked for
    int     wordCount = 0;
    if (params->wordsToLM) {
        QStringList wordsLines = readFile(params->wordFile).split("\n");    // Read file and split lines
        for (int i = 0; i < wordsLines.count(); i++) {
            wordsLines[i] = wordsLines[i].trimmed();
            if (wordsLines.isEmpty())  continue;                            // Skip empty lines
            if (!unigram.contains(wordsLines[i])) {
                unigram[wordsLines[i]] = 1;
                wordCount++;
            }
        }
    }

    QString out;
    out.append(QString("Language model created by %1 on %2.\n").arg(QDateTime::currentDateTime().toString(Qt::TextDate)).arg(APP_NAME));
    out.append("Based on QuickLM : http://www.speech.cs.cmu.edu/tools/lm.html - Carnegie Mellon University.\n");
    out.append("Copyright (C) 2020 Bruno ANSELME <be.root@free.fr>.\n\n");
    if (params->wordsToLM)
        out.append(QString("Tried to add vocabulary words, %1 were new words.\n").arg(wordCount));
    out.append(QString("This model based on a corpus of %1 sentences and %2 words.\n").arg(sent_cnt).arg(unigram.count()));
    out.append(QString("The (fixed) discount mass is %1.\n\n").arg(discount));

    // Compute counts
    qreal     unisum = 0.0;
    QMapIterator<QString, int> it_uni(unigram);     // Create iterator for future loops
    QMapIterator<QString, int> it_bi(bigram);
    QMapIterator<QString, int> it_tri(trigram);
    it_uni.toFront();               // Rewind unigram iterator
    while (it_uni.hasNext()) {
        it_uni.next();
        unisum  += it_uni.value();
    }
    out.append("\\data\\\n");
    out.append(QString("ngram 1=%1\n").arg(unigram.count()));
    if (bigram.count())
        out.append(QString("ngram 2=%1\n").arg(bigram.count()));
    if (trigram.count())
        out.append(QString("ngram 3=%1\n\n").arg(trigram.count()));

    // Compute unigrams probs
    QMap<QString,   double>    uniprob;
    it_uni.toFront();               // Rewind unigram iterator
    while (it_uni.hasNext()) {
        it_uni.next();
        uniprob[it_uni.key()]   = (it_uni.value() / unisum) * deflator;
    }
    // Compute unigrams alphas
    QMap<QString,   double>    alpha;
    it_uni.toFront();               // Rewind unigram iterator
    while (it_uni.hasNext()) {
        it_uni.next();
        double sum_denom = 0.0;
        it_bi.toFront();            // Rewind bigram iterator
        while (it_bi.hasNext()) {
            it_bi.next();
            QStringList words = it_bi.key().split(" ");
            if (words[0] == it_uni.key()) {
                sum_denom += uniprob[words[1]];
            }
        }
        alpha[it_uni.key()] = discount / (1.0 - sum_denom);
    }
    // Add unigrams to output string
    out.append("\\1-grams:\n");
    it_uni.toFront();               // Rewind unigram iterator
    while (it_uni.hasNext()) {
        it_uni.next();
        out.append(QString("%1 %2 %3\n")
                .arg(log(uniprob[it_uni.key()])/log_10,0,'f',4)
                .arg(it_uni.key())
                .arg(log(alpha[it_uni.key()])/log_10,0,'f',4)
        );
    }

    // Compute bigrams probs
    QMap<QString,   double>    biprob;
    it_bi.toFront();                // Rewind bigram iterator
    while (it_bi.hasNext()) {
        it_bi.next();
        QStringList words = it_bi.key().split(" ");
        biprob[it_bi.key()]   = (it_bi.value() * deflator) / unigram[words[0]];
    }
    // Compute bigrams alphas
    QMap<QString,   double>    bialpha;
    it_bi.toFront();                // Rewind bigram iterator
    while (it_bi.hasNext()) {
        it_bi.next();
        double sum_denom = 0.0;
        it_tri.toFront();           // Rewind trigram iterator
        while (it_tri.hasNext()) {
            it_tri.next();
            QStringList words = it_tri.key().split(" ");
            QString w2w3= words[1]+" "+words[2];
            if (w2w3 == it_bi.key()) {
                sum_denom += biprob[w2w3];
            }
        }
        bialpha[it_bi.key()] = discount / (1.0 - sum_denom);
    }
    // Add bigrams to output string
    if (bigram.count()) {
        out.append("\n\\2-grams:\n");
        it_bi.toFront();            // Rewind bigram iterator
        while (it_bi.hasNext()) {
            it_bi.next();
            out.append(QString("%1 %2 %3\n")
                    .arg(log(biprob[it_bi.key()])/log_10,0,'f',4)
                    .arg(it_bi.key())
                    .arg(log(bialpha[it_bi.key()])/log_10,0,'f',4)
            );
        }
    }

    // Add trigrams to output string
    if (trigram.count()) {
        out.append("\n\\3-grams:\n");
        it_tri.toFront();           // Rewind trigram iterator
        while (it_tri.hasNext()) {
            it_tri.next();
            QStringList words = it_tri.key().split(" ");
            QString w1w2= words[0]+" "+words[1];
            out.append(QString("%1 %2\n")
                    .arg(log((trigram[it_tri.key()]*deflator)/bigram[w1w2])/log_10,0,'f',4)
                    .arg(it_tri.key())
            );
        }
    }

    out.append("\n\\end\\\n");  // End file
    writeFile(params->lmFile, out);
}
#undef ADD_GRAM

//#include <QProcess>           // used with external language modelers (commented below)
//---------------------------------------------------------------------
// Commented code below stay here for archive. See lmtools/readme-tools.txt for more informations.
//---------------------------------------------------------------------
// Old version with QuickLM
// quick_lm.pl found here : http://www.speech.cs.cmu.edu/tools/download/quick_lm.pl
// Installed in lmtools
// Launch script 'quick_lm.pl'. Build language model. Final output : corpus.lm
/*
void LanguageModeler::runPerlQuickLM() {
    QProcess proc;
    proc.setWorkingDirectory(params->langPath.absolutePath());
    QStringList args;
    args << QString("%1/quick_lm.pl").arg(params->appPath.absolutePath());
    args << QString("-s %1").arg(params->corpusFile);
//    args << QString("-w %1").arg(params->wordFile);       // useless, add unused single words to corpus
    proc.start("/bin/perl", args);
    proc.waitForFinished();
    QString out = proc.readAllStandardError();
    proc.close();
    QFile::remove(params->lmFile);
    QFile::rename(QString("%1.arpabo").arg(params->corpusFile), params->lmFile);
}
*/
//---------------------------------------------------------------------
// Old version with CMU LMTools V2
// Needs CMU-Cam_Toolkit_v2.tar.gz found here : http://www.speech.cs.cmu.edu/SLM/toolkit.html
// Installed in lmtools
// Run CMU LMTools. Launch script 'makelm.sh'. Build language model. Final output : corpus.lm
/*
void LanguageModeler::runLMToolsV2() {
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    QProcess proc;
    proc.setWorkingDirectory(params->langPath.absolutePath());
    QStringList args;
    args << QString("%1/makelm.sh").arg(params->appPath.absolutePath());
    args << QString("-p %1").arg("corpus");
    args << QString("-d %1").arg(params->appPath.absolutePath());
    args << QString("-n %1").arg(params->ngramLevel);
    proc.start("/bin/bash", args);
    proc.waitForFinished();
    QString out = proc.readAllStandardError();
    proc.close();
    QApplication::restoreOverrideCursor();
}
*/
//---------------------------------------------------------------------
// Shell script makelm.sh used by LanguageModeler::runLMToolsV2()
/*
#!/bin/bash

NAME="newcorpus"
TOOLDIR="."
NGRAM="2"

while getopts "p:d:n:?" opt; do
        case "$opt" in
        \?) echo "Usage : $0 [OPTIONS]"
                echo "Build a language model from a text corpus with LMTools"
                echo "    Options : -p fileprefix"
                echo "    Options : -d \"lmtools/directory/path\""
                echo "    Options : -n n-gram level (1 or more)"
                exit 0
                ;;
        p)  NAME=${OPTARG}  ;;
        d)  TOOLDIR=${OPTARG}  ;;
        n)  NGRAM=${OPTARG}  ;;
        esac
done

# Create ccs file
echo "<s>" > $NAME.ccs

# Compute the word unigram counts
(cat $NAME.txt ; echo "<s>" ; echo "</s>") | $TOOLDIR/text2wfreq > $NAME.wfreq

# Convert the word unigram counts into a vocabulary consisting of the 20,000 most common words
cat $NAME.wfreq | $TOOLDIR/wfreq2vocab -top 20000 > $NAME.vocab

# Generate a binary id n-gram of the training text, based on this vocabulary
cat $NAME.txt | $TOOLDIR/text2idngram -n $NGRAM -vocab $NAME.vocab -buffer 100 -temp ~/tmp/ > $NAME.idngram

# Convert the idngram into an arpa format language model
# Output format : -arpa $NAME.lm or -binary $NAME.binlm
# The discounting strategy and its parameters are specified by the -linear, -absolute, -good_turing and -witten_bell options.
$TOOLDIR/idngram2lm -idngram $NAME.idngram -vocab $NAME.vocab -n $NGRAM -arpa $NAME.lm -witten_bell -context $NAME.ccs

# Delete temporary files
rm -f $NAME.ccs $NAME.wfreq $NAME.vocab $NAME.idngram
*/
//---------------------------------------------------------------------
