/* main.cpp  -- Main C++ program for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*! \mainpage NoComprendo Main Page
 * \section class_sec Main classes brief description.
 *  Command class contains commands types and basics command to text conversion.
 *
 *  CommandSet class link utterances to commands in a single set.
 *
 *  Comprendo class is the main application engine. It handles a list of commands sets, runs pocketsphinx voice recognition.
 *  Execute all commands but metas.
 *
 *  EditVocabulary, EditCommand, EditSet and SelectSet are configuration dialogs.
 *
 *  All application's global parameters are stored in t_params structure. This structure is sent to all other classes that need it.
 *
 *  Interface with libxdo can be found in guicontrol.cpp.
 *
 *  Interface with eSpeak can be found in speak.cpp.
 *
 */

#include <iostream>
#include <QApplication>
#include <QCommandLineParser>
#include <QTranslator>
#include <QMessageBox>
#include <QSettings>
#include "mainwindow.h"
#include "app_version.h"
#include "params.h"
#include "misc.h"
#include "guicontrol.h"
#include "speak.h"
#include "langchooser.h"

int main(int argc, char *argv[])
{
    t_params        params;
    QApplication    app(argc, argv);
    app.setOrganizationName(APP_ORG);
    app.setOrganizationDomain(APP_MAIL);
    app.setApplicationName(APP_NAME);
    app.setApplicationVersion(APP_VERSION);

    // Set default language
    params.lang = QLocale::system().name();         // environment variable LANGUAGE
    // Check for an existing default language in settings
    QSettings   settings(APP_ORG, APP_NAME);
    QString     defLang = settings.value("defLang", "").toString();
    if (defLang == "") {
        LangChooser    dial;                       // Language selector
        int dialogCode = dial.exec();
        if (dialogCode == QDialog::Accepted) {
            params.lang = dial.getLang();
            settings.setValue("defLang", params.lang);
        } else {
            exit(0);
        }
    } else {
        params.lang = defLang;
    }

    // Load translations
    QTranslator qtTranslator;
    if (params.lang != "en_US") {
        if (qtTranslator.load(":/nocomprendo_" + params.lang)) {
            app.installTranslator(&qtTranslator);
        } else {
            std::cerr << "Translation file not found for " << params.lang.toStdString().c_str() << "\n";
        }
    }

    // Parse command line options
    QCommandLineParser parser;
    parser.addHelpOption();
    parser.setApplicationDescription(QObject::tr("Voice command for Linux/X.org.\nBased on Qt and PocketSphinx."));
    parser.addVersionOption();
    QCommandLineOption authorOption(QStringList()  << "a" << "author",          QObject::tr("Show author informations."));
    parser.addOption(authorOption);
    QCommandLineOption licenseOption(QStringList() << "i" << "license",         QObject::tr("Show license informations."));
    parser.addOption(licenseOption);
    QCommandLineOption libverOption(QStringList()  << "b" << "libs",            QObject::tr("Show librairies versions."));
    parser.addOption(libverOption);
    QCommandLineOption pocketOption(QStringList()  << "p" << "pocket",          QObject::tr("Full pocketsphinx log."));
    parser.addOption(pocketOption);
    QCommandLineOption wordOption(QStringList()    << "w" << "words",           QObject::tr("Trace recognized words."));
    parser.addOption(wordOption);
    QCommandLineOption lmOption(QStringList()      << "l" << "languageModel",   QObject::tr("Show language model file."));
    parser.addOption(lmOption);
    // Process the actual command line arguments given by the user
    parser.process(app);
    params.pocketTrace  = parser.isSet("pocket");
    params.wordTrace    = parser.isSet("words");

    // Set application parameters. Set different directories for development and installed version
    if (QApplication::applicationDirPath()=="/usr/bin") {       // Installation directories
        params.development      = false;
        params.appPath          = QString("/usr/share/%1").arg(APP_STR);
        params.pocketPath       = QString(POCKET_PATH).append(params.lang);
        params.langPath         = QDir::homePath().append("/.config/").append(APP_STR).append("/").append(params.lang);
        params.lockFile         = QDir::homePath().append("/.config/").append(APP_STR).append("/").append(APP_LOCKFILE);
    } else {                                                    // Development directories
        params.development      = true;
        params.appPath          = QApplication::applicationDirPath().append("/lmtools");
        params.pocketPath       = QApplication::applicationDirPath().append("/pocketsphinx/").append(params.lang);
        params.langPath         = QApplication::applicationDirPath().append("/langs/").append(params.lang);
        params.lockFile         = QApplication::applicationDirPath().append("/").append(APP_LOCKFILE);
    }
    params.wordFile             = QString("%1/words.txt").arg(params.langPath.absolutePath());
    params.corpusFile           = QString("%1/corpus.txt").arg(params.langPath.absolutePath());
    params.dictFile             = QString("%1/words.dict").arg(params.langPath.absolutePath());
    params.lmFile               = QString("%1/corpus.lm").arg(params.langPath.absolutePath());
    params.mouseFile            = QString("%1/mouseto.txt").arg(params.langPath.absolutePath());
    params.langShort            = params.lang.left(2);
    params.autoStart            = false;
    params.removeBadUtterance   = false;
    params.osdOn                = false;
    params.osdAll               = false;
    params.osdDuration          = 3000;
    params.osdPosition          = QPoint(100,250);
    params.osdSize              = QSize(450, 40);
    params.speak                = true;
    params.gender               = 2;
    params.age                  = 45;

    // Check for lockfile
    if (QFile::exists(params.lockFile)) {           // Check for an already running instance
        QMessageBox::information(
                    nullptr,
                    APP_STR,
                    QObject::tr("%1 is already running.\n\nKill running process and/or remove log file :\n%2")
                        .arg(APP_NAME).arg(params.lockFile),
                    QMessageBox::Ok);
        exit(-1);
    }

    // Run command line options and exit
    if (parser.isSet("author")) {
        std::cout << QObject::tr("%1 was written by:\n\t%2 <%3>\n").arg(APP_STR).arg(APP_AUTHOR).arg(APP_MAIL).toStdString().c_str();
        std::cout << QObject::tr("\tWeb site : %1\n").arg(APP_SITE).toStdString().c_str();
        exit(EXIT_SUCCESS);
    }
    if (parser.isSet("license")) {
        std::cout << readFile(":/license-gpl-3.0.txt").toStdString().c_str();
        exit(EXIT_SUCCESS);
    }
    if (parser.isSet("languageModel")) {
        std::cout << readFile(params.lmFile).toStdString().c_str();
        exit(EXIT_SUCCESS);
    }
    if (parser.isSet("libs")) {
        std::cout << "Librairies version\n";
        std::cout << "\tpocketsphinx  : 5prealpha (embedded)\n";
        std::cout << "\tlibxdo        : " << guiVersion() << " (external)\n";
        std::cout << "\tlibespeak     : " << eSpeakVersion() << " (external) \n";
        exit(EXIT_SUCCESS);
    }

    // Create and run main window
    MainWindow w(params);
    if (QApplication::applicationDirPath()=="/usr/bin") {                       // Installation directories
        w.setWindowTitle(QString("%1").arg(APP_NAME));                          // Only application name
    } else {                                                                    // Development directories
        w.setWindowTitle(QString("%1 - %2").arg(APP_NAME).arg(APP_VERSION));    // Application name and version number for developper
    }
    w.show();
    return app.exec();
}
