/* mainwindow.cpp  -- MainWindow class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      mainwindow.cpp
 * @author    Bruno ANSELME
 * @brief     MainWindow class definition
 * @details   Application's main window.
 */

#include <signal.h>
#include <sys/socket.h>
#include <unistd.h>
#include <iostream>
#include <QTextStream>
#include <QKeyEvent>
#include <QScrollBar>
#include <QSettings>
#include <QFileDialog>
#include <QFileSystemModel>
#include <QAbstractButton>
#include <QMessageBox>
#include <QAction>
#include "mainwindow.h"
#include "editset.h"
#include "misc.h"
#include "editvocabulary.h"
#include "selectset.h"
#include "guicontrol.h"
#include "speak.h"
#include "settings.h"
#include "app_version.h"
#include "zoombutton.h"
#include "languagemodeler.h"
#include <QDebug>

static int          sigtermFd[2];
static int          sigintFd[2];
static int          sighupFd[2];

static int setup_unix_signal_handlers()
{
    struct sigaction sigterm;
    sigterm.sa_handler = MainWindow::termSignalHandler;
    sigemptyset(&sigterm.sa_mask);
    sigterm.sa_flags |= SA_RESTART;
    if (sigaction(SIGTERM, &sigterm, 0))
       return SIGTERM;
    struct sigaction sigint;
    sigint.sa_handler = MainWindow::intSignalHandler;
    sigemptyset(&sigint.sa_mask);
    sigint.sa_flags |= SA_RESTART;
    if (sigaction(SIGINT, &sigint, 0))
       return SIGINT;
    struct sigaction sighup;
    sighup.sa_handler = MainWindow::hupSignalHandler;
    sigemptyset(&sighup.sa_mask);
    sighup.sa_flags |= SA_RESTART;
    if (sigaction(SIGHUP, &sighup, 0))
       return SIGHUP;
    return 0;
}

MainWindow::MainWindow(t_params &p, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    suspended(false),
    trashNext(false),
    lastOk(true),
    shortcutFormat(CommandSet::QtShortcut)
{
    // Connect TERM signal handlers
    if (::socketpair(AF_UNIX, SOCK_STREAM, 0, sigtermFd))
       qFatal("Couldn't create TERM socketpair");
    snTerm = new QSocketNotifier(sigtermFd[1], QSocketNotifier::Read, this);
    connect(snTerm, SIGNAL(activated(int)), this, SLOT(handleSigTerm()));
    // Connect INT signal handlers
    if (::socketpair(AF_UNIX, SOCK_STREAM, 0, sigintFd))
       qFatal("Couldn't create INT socketpair");
    snInt = new QSocketNotifier(sigintFd[1], QSocketNotifier::Read, this);
    connect(snInt, SIGNAL(activated(int)), this, SLOT(handleSigInt()));
    // Connect HUP signal handlers
    if (::socketpair(AF_UNIX, SOCK_STREAM, 0, sighupFd))
       qFatal("Couldn't create HUP socketpair");
    snHup = new QSocketNotifier(sighupFd[1], QSocketNotifier::Read, this);
    connect(snHup, SIGNAL(activated(int)), this, SLOT(handleSigHup()));

    setup_unix_signal_handlers();

    params      = p;
    aboutDialog = nullptr;
    reminder    = nullptr;
    timerid     = 0;
    lineCount   = 0;
    writeFile(params.lockFile,QString("%1 lock file\nPID: %2\n").arg(APP_NAME).arg(QCoreApplication::applicationPid()));
    ui->setupUi(this);
    checkFirstTime(params);
    readSettings();
    what        = new Comprendo(&params, this);
    updateCommands();
    updateVocabulary();
    ui->btSuspend->setEnabled(false);
#ifdef ENABLE_ESPEAK
    initSpeak();
    setVoice("es", params.gender, params.age, 0);
    if (params.speak) sayIt(QString("No  comprendo.").toStdString().c_str());
    setVoice(params.langShort.toStdString().c_str(), params.gender, params.age, params.voiceVariant);
#endif
    connect(ui->btQuit,         SIGNAL(clicked()),      this, SLOT(askForClose()));
    connect(ui->btPlay,         SIGNAL(toggled(bool)),  this, SLOT(listen(bool)));
    connect(ui->btSuspend,      SIGNAL(toggled(bool)),  this, SLOT(suspend(bool)));
    connect(ui->btClear,        SIGNAL(clicked()),      this, SLOT(clearConsole()));
    connect(ui->btVocabulary,   SIGNAL(clicked()),      this, SLOT(editVocabulary()));
    connect(ui->btSets,         SIGNAL(clicked()),      this, SLOT(editSets()));
    connect(ui->btConfig,       SIGNAL(clicked()),      this, SLOT(doSettings()));
    connect(ui->btReminder,     SIGNAL(clicked()),      this, SLOT(openReminder()));
    connect(ui->btAbout,        SIGNAL(clicked()),      this, SLOT(about()));
    connect(what,               SIGNAL(statusListening(bool)),                                  this, SLOT(listeningStatus(bool)));
    connect(what,               SIGNAL(metaCommand(Command::metaType)),                         this, SLOT(execMeta(Command::metaType)));
    connect(ui->tableCommands->horizontalHeader(), SIGNAL(customContextMenuRequested(QPoint)),  this, SLOT(tableContextMenu(QPoint)));
    if (params.autoStart)  timerstart = startTimer(1000);
    listener        = new Listener(what);
    listenState     = new QLabel(tr("Stopped"));
    osd = new OSDWindow(&params);
    blinker = new MouseBlink(&params);
    QFont font1;
    font1.setBold(true);
    listenState->setFont(font1);
    statusBar()->addPermanentWidget(listenState);
    delayedMethod   = nullptr;
    runningMethod   = nullptr;
    tray            = nullptr;
    isHidden        = params.hideInTaskBar;
    if (isHidden)
        setWindowFlags(windowFlags() | Qt::Tool);
    show();
    activateWindow();
    runSysTray();
    if (tray) tray->setMenus((timerid != 0), suspended);
    // Test new language modeler
    LanguageModeler lm(&params);
    lm.quickLM();                       // Run new LM code
}

MainWindow::~MainWindow()
{
    writeSettings();
    if (listener->isRunning()) listener->wait();
    listener->quit();
    if (what->isInitialized()) {
        listen(false);
    }
#ifdef ENABLE_ESPEAK
    if (params.speak) sayIt(tr("Good bye").toStdString().c_str());
#endif
    delete what;
    delete osd;
    delete blinker;
    delete ui;
    QFile::remove(params.lockFile);
}

void MainWindow::termSignalHandler(int) {   char a = 1; ::write(sigtermFd[0], &a, sizeof(a));   }
void MainWindow::intSignalHandler(int)  {   char a = 1; ::write(sigintFd[0],  &a, sizeof(a));    }
void MainWindow::hupSignalHandler(int)  {   char a = 1; ::write(sighupFd[0],  &a, sizeof(a));    }
void MainWindow::handleSigTerm() {
    snTerm->setEnabled(false);
    char tmp;
    ::read(sigtermFd[1], &tmp, sizeof(tmp));
    std::cerr << APP_STR << " : SIGTERM received, exiting program.\n";
    askForClose();
    snTerm->setEnabled(true);
}
void MainWindow::handleSigInt() {
    snInt->setEnabled(false);
    char tmp;
    ::read(sigintFd[1], &tmp, sizeof(tmp));
    std::cerr << APP_STR << " : SIGINT received, exiting program.\n";
    askForClose();
    snInt->setEnabled(true);
}
void MainWindow::handleSigHup() {
    snHup->setEnabled(false);
    char tmp;
    ::read(sighupFd[1], &tmp, sizeof(tmp));
    std::cerr << APP_STR << " : SIGHUP received, exiting program.\n";
    askForClose();
    snHup->setEnabled(true);
}

void MainWindow::clearConsole() {
    ui->textOutput->clear();
    lineCount   = 0;
}

void MainWindow::removeFirstLine() {
    QTextCursor cursor = ui->textOutput->textCursor();
    cursor.movePosition(QTextCursor::Start);
    cursor.movePosition(QTextCursor::Down, QTextCursor::KeepAnchor, 1);
    cursor.removeSelectedText();
    ui->textOutput->moveCursor(QTextCursor::End);
    lineCount--;
}

void MainWindow::askForClose() {
    if (params.ask2Quit) {
        QMessageBox::StandardButton reply = QMessageBox::question(nullptr, tr("Exit"), tr("Do you want to quit %1 ?").arg(APP_NAME),
                QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
        if (reply == QMessageBox::No)  return;
    }
    close();
}

void MainWindow::raiseWindow() {        // This function comes into conflict with desktop manager
    setWindowState( (windowState() & ~Qt::WindowMinimized));
    show();
    activateWindow();
    raise();
}

// Run system tray if available. Connect menus signals.
void MainWindow::runSysTray() {
    if (params.useSysTray) {
        if (!tray) {
            tray = new TrayIcon(&params);
            connect(tray,   SIGNAL(trayStart()),        ui->btPlay,     SLOT(click()));
            connect(tray,   SIGNAL(trayStop()),         ui->btPlay,     SLOT(click()));
            connect(tray,   SIGNAL(traySuspend()),      ui->btSuspend,  SLOT(click()));
            connect(tray,   SIGNAL(trayQuit()),         ui->btQuit,     SLOT(click()));
            connect(tray,   SIGNAL(trayShow()),         this,           SLOT(raiseWindow()));
        }
    } else {
        if (tray) {
            delete tray;        // disconnect all signals
            tray    = nullptr;
        }
    }
}

// Change SystemTrayIcon depending on listening and suspend states
void MainWindow::listeningStatus(bool b) {
    if (b) {
        statusBar()->showMessage(tr("Listening..."));
        if (tray) {
            if (suspended)  tray->setStatus(TrayIcon::ListenSuspend);
            else            tray->setStatus(TrayIcon::Listening);
        }
    } else {
        statusBar()->showMessage(tr("Ready..."));
        if (tray) {
            if (suspended)  tray->setStatus(TrayIcon::Suspended);
            else            tray->setStatus(TrayIcon::Ready);
        }
    }
}

void MainWindow::closeEvent(QCloseEvent *event) {   // closeEvent can be sent application by environment
    osd->close();
    QApplication::quit();       // close all opened dialogs
    event->accept();
}

// First time actions : copy configuration files, default command sets, create settings file
void MainWindow::checkFirstTime(t_params &p) {
    if (p.development) return;
    if (!p.langPath.exists()) {
        QDir().mkpath(p.langPath.absolutePath());
        QDir    source  = QString("%1/defsets/%2").arg(p.appPath.absolutePath()).arg(params.lang);
        source.setNameFilters(QStringList() << "*.cmd" << "*.lm" << "words.dict" << "*.txt");
        source.setFilter(QDir::Files);

        QSettings settings(APP_ORG, APP_NAME);
        settings.remove(params.langShort);
        settings.beginGroup(params.langShort);

        QStringList files   = source.entryList();
        QRegExp     cmdpat  = QRegExp("\\.cmd$");
        QListIterator<QString> filename(files);
        while (filename.hasNext()) {                // Copy all files
            QString name = filename.next();
            QFile::copy(QString("%1/%2").arg(source.absolutePath()).arg(name),
                        QString("%1/%2").arg(p.langPath.absolutePath()).arg(name));
            if (name.contains(cmdpat))              // Add commands sets to settings
                settings.setValue(name.left(name.count()-4), name);
        }
        settings.endGroup();
    }
}

void MainWindow::about()
{
    if (aboutDialog == nullptr) {
        aboutDialog = new AboutDialog(params.lang, this);
    }
    aboutDialog->show();
    aboutDialog->raise();
    aboutDialog->activateWindow();
}

void MainWindow::showSet(bool b)        { ui->tableCommands->setColumnHidden(COL_SET,       !b); }
void MainWindow::showType(bool b)       { ui->tableCommands->setColumnHidden(COL_TYPE,      !b); }
void MainWindow::showCommand(bool b)    { ui->tableCommands->setColumnHidden(COL_COMMAND,   !b); }

void MainWindow::toQt(bool b) {
    if (b) shortcutFormat = CommandSet::QtShortcut;
    updateCommands();
}
void MainWindow::toXdo(bool b) {
    if (b) shortcutFormat = CommandSet::XdoShortcut;
    updateCommands();
}

void MainWindow::tableContextMenu(QPoint p) {
    QMenu myMenu;
    QAction    *action;

    action = new QAction(tr("Type"), nullptr);
    action->setCheckable(true);
    action->setChecked(!ui->tableCommands->isColumnHidden(COL_TYPE));
    connect(action,     SIGNAL(triggered(bool)), this,  SLOT(showType(bool)));
    myMenu.addAction(action);

    action = new QAction(tr("Command"), nullptr);
    action->setCheckable(true);
    action->setChecked(!ui->tableCommands->isColumnHidden(COL_COMMAND));
    myMenu.addAction(action);
    connect(action,     SIGNAL(triggered(bool)), this,  SLOT(showCommand(bool)));

    action = new QAction(tr("Set"), nullptr);
    action->setCheckable(true);
    action->setChecked(!ui->tableCommands->isColumnHidden(COL_SET));
    connect(action,     SIGNAL(triggered(bool)), this,  SLOT(showSet(bool)));
    myMenu.addAction(action);

    myMenu.addSeparator();

    QActionGroup *shortcutGroup = new QActionGroup(this);
    action = new QAction(tr("Qt shortcuts"), nullptr);
    action->setCheckable(true);
    action->setChecked(shortcutFormat == CommandSet::QtShortcut);
    shortcutGroup->addAction(action);
    myMenu.addAction(action);
    connect(action,     SIGNAL(triggered(bool)), this, SLOT(toQt(bool)));

    action = new QAction(tr("Xdo shortcuts"), nullptr);
    action->setCheckable(true);
    action->setChecked(shortcutFormat == CommandSet::XdoShortcut);
    shortcutGroup->addAction(action);
    myMenu.addAction(action);
    connect(action,     SIGNAL(triggered(bool)), this, SLOT(toXdo(bool)));

    myMenu.exec(ui->tableCommands->mapToGlobal(p));
}

void MainWindow::updateVocabulary() {
    ui->textVocabulary->setPlainText(readFile(params.dictFile));
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::openReminder() {
    if (!reminder) {
        reminder = new Reminder(&params, what);
        connect(reminder, SIGNAL(rejected()), this, SLOT(closeReminder()));
        connect(reminder, SIGNAL(accepted()), this, SLOT(closeReminder()));
        reminder->show();
    }
}

void MainWindow::closeReminder() {
    disconnect(reminder, SIGNAL(rejected()), this, SLOT(closeReminder()));
    disconnect(reminder, SIGNAL(accepted()), this, SLOT(closeReminder()));
    reminder->deleteLater();
    reminder    = nullptr;
}

void MainWindow::execMeta(Command::metaType m) {            // Meta-commands are executed here
    switch (m) {
    case Command::Quit :
        ui->btQuit->click();
        break;
    case Command::Stop :
        ui->btPlay->toggle();
        suspended  = false;
        break;
    case Command::Suspend :
        if (!ui->btSuspend->isChecked())
            ui->btSuspend->click();
        break;
    case Command::Resume :
        if (ui->btSuspend->isChecked())
            ui->btSuspend->click();
        break;
    case Command::Repeat :
        what->repeatLastCommand(suspended);
        break;
    case Command::Vocabulary:
        if (!runningMethod)
           delayedMethod = &MainWindow::editVocabulary;
        break;
    case Command::CommandSets :
        if (!runningMethod)
            delayedMethod = &MainWindow::editSets;
        break;
    case Command::Settings:
        if (!runningMethod)
            delayedMethod = &MainWindow::doSettings;
        break;
    case Command::Reminder:
        openReminder();
        break;
    case Command::Raise:
        raiseWindow();
        break;
    case Command::Minimize:
        setWindowState( (windowState() | Qt::WindowMinimized));
        break;
    case Command::MouseShow:
        blinker->start();
        break;
    }
}

void MainWindow::removeLastOutput() {
    if (!suspended) {
        QTextCursor cursor = ui->textOutput->textCursor();
        cursor.movePosition(QTextCursor::End);
        cursor.select(QTextCursor::LineUnderCursor);
        cursor.removeSelectedText();
        cursor.deletePreviousChar();                                    // Added to trim the newline char when removing last line
        ui->textOutput->setTextCursor(cursor);
        lineCount--;
    }
}

void MainWindow::utteranceReceived(QString utterance) {                           // Called after signal Comprendo::resultReady
    if (!trashNext) {
        bool wassuspended   = suspended;
        bool cmdok = what->doCommand(utterance, suspended);
        if ((!lastOk) && (params.removeBadUtterance)) removeLastOutput();
        lastOk = cmdok;
        if (!utterance.isEmpty() && ((!suspended) || (!wassuspended))) {     // Stop displaying while suspended
            if (params.wordTrace) {
                std::cout << utterance.toStdString().c_str() << "\n";
                std::cout.flush();
            }
            ui->textOutput->setFontWeight(QFont::Normal);
            if (!cmdok) ui->textOutput->setTextColor(Qt::gray);         // unknown utterance color
            else        ui->textOutput->setTextColor(Qt::black);        // valid utterance color
            ui->textOutput->append(utterance);
            ui->textOutput->moveCursor(QTextCursor::End);
            ui->textOutput->moveCursor(QTextCursor::StartOfBlock);
            ui->textOutput->setTextColor(Qt::black);                    // back to default color
            lineCount++;
            ui->leRepeat->setText(what->getLastCommand());
            if ((cmdok) && (params.osdOn))
                osd->setFlashingText(utterance);
            else if ((params.osdOn) && (params.osdAll))
                osd->setUnknown(utterance);
            if (delayedMethod) {                    // Methods are application's dialogs
                runningMethod = delayedMethod;      // Copy method to an other pointer
                delayedMethod = nullptr;            // Clear delayedMethod pointer first to avoid loop
                (this->*runningMethod)();           // Run the delayed method (dialog) from the other pointer
                runningMethod    = nullptr;         // Clear runningMethod after execution
            }
            while (lineCount > params.maxlines)
                removeFirstLine();
        }
    } else trashNext = false;
}

void MainWindow::timerEvent(QTimerEvent *t){
    if ((what) && (t->timerId()==timerid)) {
        if (!what->isZooming())         // while zooming, listening is done by ZoomButton object
            listener->start();
    }
    if (t->timerId()==timerstart) {     // Timer for comprendo/sphinx first init
        killTimer(timerstart);
        timerstart = 0;
        if (!what->isInitialized()) ui->btPlay->toggle();
    }
}

void MainWindow::suspend(bool b) {
    ui->textOutput->setFontWeight(QFont::Bold);
    if (b) {
        suspended = true;
        listenState->setText(tr("Suspended"));
        if (params.osdOn) osd->setFlashingText(tr("Suspended"));
        if (tray)  tray->setStatus(TrayIcon::Suspended);
#ifdef ENABLE_ESPEAK
        if (params.speak) {
            sayIt(tr("I suspend listening.").toStdString().c_str());
            trashNext = true;
        }
#endif
    } else {
        suspended = false;
        listenState->setText(tr("Listening"));
        if (params.osdOn) osd->setFlashingText(tr("Listening"));
        if (tray)  tray->setStatus(TrayIcon::Ready);
#ifdef ENABLE_ESPEAK
        if (params.speak) {
            sayIt(tr("I resume listening.").toStdString().c_str());
            trashNext = true;
        }
#endif
    }
    ui->textOutput->setFontWeight(QFont::Normal);
    lastOk = true;
    if (tray) tray->setMenus((timerid != 0), suspended);
}

void MainWindow::listen(bool b) {
    if (b) {
        if (!what->initPSphinx()) {
            ui->btPlay->setChecked(false);
            return;
        }
#ifdef ENABLE_ESPEAK
        setVoice(params.langShort.toStdString().c_str(), params.gender, params.age, params.voiceVariant);
        if (params.speak) sayIt(tr("I am ready, listening.").toStdString().c_str());
#endif
        ui->textOutput->setFontWeight(QFont::Bold);
        ui->textOutput->append(tr("Ready..."));
        lineCount++;
        if (params.osdOn) osd->setFlashingText(tr("Ready..."));
        ui->textOutput->setFontWeight(QFont::Normal);
        timerid = startTimer(10);
        connect(listener,     SIGNAL(resultReady(QString)),   this,   SLOT(utteranceReceived(QString)));
        listenState->setText(tr("Listening"));
        ui->btSuspend->setEnabled(true);
        ui->btSuspend->setChecked(false);
        if (tray)  tray->setStatus(TrayIcon::Ready);
    } else {
        if (listener->isRunning()) listener->wait();
        listener->quit();
        disconnect(listener,     SIGNAL(resultReady(QString)),   this,   SLOT(utteranceReceived(QString)));
        killTimer(timerid);
        suspended = false;
        timerid = 0;
        what->closePSphinx();
        listenState->setText(tr("Stopped"));
        statusBar()->showMessage("");
        ui->btSuspend->setChecked(false);
        ui->btSuspend->setEnabled(false);
        ui->textOutput->setFontWeight(QFont::Bold);
        ui->textOutput->append(tr("Stopped"));
        lineCount++;
        ui->textOutput->setFontWeight(QFont::Normal);
        if (params.osdOn) osd->setFlashingText(tr("Stopped"));
        if (tray)  tray->setStatus(TrayIcon::Stopped);
#ifdef ENABLE_ESPEAK
        if (params.speak) sayIt(tr("I stop now.").toStdString().c_str());
#endif
    }
    if (tray) tray->setMenus((timerid != 0), suspended);
    lastOk = true;
}

void MainWindow::updateCommands() {         // Fill GUI table with Comprendo object data
    ui->tableCommands->clearContents();
    ui->tableCommands->setRowCount(0);
    ui->tableCommands->setSortingEnabled(false);
    what->fillTable(ui->tableCommands, shortcutFormat);
    ui->tableCommands->setSortingEnabled(true);
    ui->tableCommands->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    ui->tableCommands->horizontalHeader()->setContextMenuPolicy(Qt::CustomContextMenu);
    ui->tableCommands->verticalHeader()->setSectionResizeMode(QHeaderView::Fixed);
    ui->tableCommands->verticalHeader()->setDefaultSectionSize(16);
}

void MainWindow::updateLanguageModel() {
    what->loadCommandSets();                // Update Comprendo with selected sets
    what->updateCorpus();                   // Build corpus with vocabulary and selected sets
    LanguageModeler lm(&params);
//    lm.runPerlQuickLM();                    // Run quick_lm.pl command line tool (build language model). Obsolete since version 1.0.14
    lm.quickLM();                           // Run home made language model builder
    updateCommands();                       // Update main window's commands list
    restartListener();                      // Corpus changed, restart PocketSphinx
    statusBar()->showMessage(tr("Language model is up to date"),5000);
}

void MainWindow::restartListener() {
    if (ui->btPlay->isChecked()) {
         if (listener->isRunning())
             listener->wait();
        listener->quit();
        disconnect(listener,    SIGNAL(resultReady(QString)),   this,   SLOT(utteranceReceived(QString)));
        timerid = 0;
        what->closePSphinx();
        if (!what->initPSphinx()) {
            ui->btPlay->setChecked(false);
            return;
        }
        timerid = startTimer(10);
        connect(listener,       SIGNAL(resultReady(QString)),   this,   SLOT(utteranceReceived(QString)));
    }
}

void MainWindow::editSets() {
    SelectSet dial(params, what);
    connect(&dial,  SIGNAL(setsUpdated()),   this,   SLOT(updateLanguageModel()));
    dial.exec();
}

void MainWindow::editVocabulary() {
    EditVocabulary dial(params, what);
    int dialogCode = dial.exec();
    if (dialogCode == QDialog::Accepted) {
        updateVocabulary();
        updateLanguageModel();
    }
}

void MainWindow::information(const QString &mes) {
    QMessageBox msgBox;
    msgBox.setWindowTitle(tr("%1 information").arg(APP_NAME));
    msgBox.setText(mes);
    msgBox.exec();
}

void MainWindow::doSettings() {
    Settings dial(&params);
    connect(&dial,   SIGNAL(sendInformation(const QString &)),     this,   SLOT(information(const QString &)));
    int dialogCode = dial.exec();
    if (dialogCode == QDialog::Accepted) {
        writeSettings();
        if (isHidden) params.useSysTray = true;             // Force systray when hidden in taskbar otherwise desktop manager kills mainWindow
        runSysTray();
        osd->changeOsd(params.osdPosition, params.osdSize);
        updateLanguageModel();                              // mouseto utterances could have changed
#ifdef ENABLE_ESPEAK
        setVoice(params.langShort.toStdString().c_str(), params.gender, params.age, params.voiceVariant);
#endif
    }
}

void MainWindow::readSettings() {
    QSettings settings(APP_ORG, APP_NAME);
    move(settings.value("mainPos", QPoint(200, 200)).toPoint());
    resize(settings.value("mainSize", QSize(600, 450)).toSize());
    ui->splitter->restoreState(settings.value("splitter").toByteArray());
    params.speak                = settings.value("speak", true).toBool();
    params.autoStart            = settings.value("autoStart", false).toBool();
    params.ask2Quit             = settings.value("ask2Quit", false).toBool();
    params.maxlines             = settings.value("maxlines", 100).toInt();
    params.removeBadUtterance   = settings.value("removeBadUtterance", false).toBool();
    params.hideInTaskBar        = settings.value("hideInTaskBar", false).toBool();
    params.useSysTray           = settings.value("useSysTray", QSystemTrayIcon::isSystemTrayAvailable()).toBool();
    params.mouseStep            = settings.value("mouseStep", 10).toInt();
    params.glideStep            = settings.value("glideStep", 2).toInt();
    params.glideRate            = settings.value("glideRate", 20).toInt();
    params.osdOn                = settings.value("osdOn", false).toBool();
    params.osdAll               = settings.value("osdAll", false).toBool();
    params.osdDuration          = settings.value("osdDuration", 3000).toInt();
    params.osdPosition          = settings.value("osdPosition", QPoint(100, 250)).toPoint();
    params.osdSize              = settings.value("osdSize", QSize(450, 40)).toSize();
    params.rFontSize            = settings.value("rFontSize", 18).toInt();
    params.rInvertColor         = settings.value("rInvertColor", false).toBool();
    params.wordsToLM            = settings.value("wordsToLM", false).toBool();
    settings.beginGroup(params.lang);
        params.gender           = settings.value("gender", 2).toInt();
        params.age              = settings.value("age", 45).toInt();
        params.voiceVariant     = settings.value("voiceVariant", 0).toInt();
    settings.endGroup();
}

void MainWindow::writeSettings() {
    QSettings settings(APP_ORG, APP_NAME);
    settings.setValue("mainPos",            pos());
    settings.setValue("mainSize",           size());
    settings.setValue("splitter",           ui->splitter->saveState());
    settings.setValue("autoStart",          params.autoStart);
    settings.setValue("ask2Quit",           params.ask2Quit);
    settings.setValue("maxlines",           params.maxlines);
    settings.setValue("hideInTaskBar",      params.hideInTaskBar);
    settings.setValue("removeBadUtterance", params.removeBadUtterance);
    settings.setValue("useSysTray",         params.useSysTray);
    settings.setValue("speak",              params.speak);
    settings.setValue("mouseStep",          params.mouseStep);
    settings.setValue("glideStep",          params.glideStep);
    settings.setValue("glideRate",          params.glideRate);
    settings.setValue("osdOn",              params.osdOn);
    settings.setValue("osdAll",             params.osdAll);
    settings.setValue("osdDuration",        params.osdDuration);
    settings.setValue("osdPosition",        params.osdPosition);
    settings.setValue("osdSize",            params.osdSize);
    settings.setValue("rFontSize",          params.rFontSize);
    settings.setValue("rInvertColor",       params.rInvertColor);
    settings.setValue("wordsToLM",          params.wordsToLM);
    settings.beginGroup(params.lang);
        settings.setValue("gender",         params.gender);
        settings.setValue("age",            params.age);
       settings.setValue("voiceVariant",    params.voiceVariant);
    settings.endGroup();
}

