/* mainwindow.h  -- MainWindow class definition for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QObject>
#include <QMainWindow>
#include <QDir>
#include <QSocketNotifier>
#include "comprendo.h"
#include "listener.h"
#include "aboutdialog.h"
#include "params.h"
#include "osd.h"
#include "reminder.h"
#include "trayicon.h"
#include "mouseblink.h"
#include "ui_mainwindow.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit    MainWindow(t_params &p, QWidget *parent = nullptr); //!< Constructor
    ~MainWindow();
    static void termSignalHandler(int unused);          //!< Unix signal SIGTERM handler.
    static void intSignalHandler(int unused);           //!< Unix signal SIGINT handler.
    static void hupSignalHandler(int unused);           //!< Unix signal SIGHUP handler.
public slots:
    void        handleSigTerm();                        //!< Qt signal handlers.
    void        handleSigInt();                         //!< Qt signal handlers.
    void        handleSigHup();                         //!< Qt signal handlers.
    void        editSets();                             //!< Start sets editor
    void        editVocabulary();                       //!< Start vocabulary editor
    void        updateVocabulary();                     //!< Reload vocabulary in vocabulary tab
    void        updateCommands();                       //!< Reload commnds in commands tab
    void        doSettings();                           //!< Start settings dialog
    void        execMeta(Command::metaType m);          //!< Run meta command (connected to Comprendo::metaCommand signal)
    void        about();                                //!< Display about dialog
    void        information(const QString &);           //!< Display an information dialog
protected slots:
    void        listen(bool);                           //!< Start/stop Listener object, update application status
    void        suspend(bool);                          //!< Suspend/resume listening, update application status
    void        utteranceReceived(QString utterance);   //!< Send utterance to Comprendo object, print utterance, update application status
    void        showType(bool b);                       //!< Show column "type" in commands list
    void        showCommand(bool b);                    //!< Show column "command" in commands list
    void        showSet(bool b);                        //!< Show column "set" in commands list
    void        tableContextMenu(QPoint);               //!< Setup and run commands list context menu
    void        toQt(bool);                             //!< Show Qt sequence in column "command"
    void        toXdo(bool);                            //!< Show KeySym sequence in column "command"
    void        updateLanguageModel();                  //!< Update corpus and run LanguageModeler
    void        openReminder();                         //!< Create Reminder object
    void        closeReminder();                        //!< Delete Reminder object
    void        listeningStatus(bool);                  //!< Change TrayIcon depending on listening and suspend states
    void        raiseWindow();                          //!< Bring MainWindow to front
    void        askForClose();                          //!< Close confirmation dialog
    void        clearConsole();                         //!< Clear console
    void        removeFirstLine();                      //!< Remove console's first line
protected:
    void        changeEvent(QEvent *e);                 //!< Handle language change
    void        timerEvent(QTimerEvent *event);         //!< Handle start timer and glide timer
    void        closeEvent(QCloseEvent *event);         //!< Clean up all opened dialogs before closing
private:
    void        checkFirstTime(t_params &p);
    void        readSettings();
    void        writeSettings();
    void        removeLastOutput();
    void        restartListener();
    void        runSysTray();
private:
    Ui::MainWindow     *ui;
    t_params            params;                             // Global parameters
    Comprendo          *what;                               // Voice command engine
    Listener           *listener;                           // Comprendo runs in a different thread called Listener
    int                 timerid, timerstart;
    bool                suspended;
    bool                trashNext;
    bool                lastOk;
    bool                isHidden;
    int                 lineCount;
    CommandSet::ShortcutFormat  shortcutFormat;             // QtShortcut or XdoShortcut
    AboutDialog        *aboutDialog;                        // aboutDialog pointer
    Reminder           *reminder;                           // Reminder dialog pointer
    QLabel             *listenState;                        // Right field in status bar (Stopped, Listening, Suspended)
    OSDWindow          *osd;                                // On screen display window
    MouseBlink         *blinker;
    TrayIcon           *tray;
    // function pointers for meta-commands launching dialogs
    void                (MainWindow::*delayedMethod)();     // Method waiting for execution
    void                (MainWindow::*runningMethod)();     // Method running
    // Signal handler data
    QSocketNotifier    *snTerm, *snInt, *snHup;
};

#endif // MAINWINDOW_H
