/* misc.cpp  -- Miscellaneous functions implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      misc.cpp
 * @author    Bruno ANSELME
 * @brief     Miscellaneous functions
 * @details   Read and write file. get desktop rectangle.
 */

#include <QFile>
#include <QTextStream>
#include <QGuiApplication>
#include <QScreen>
#include "misc.h"

QString readFile(QString fname) {
    QFile file(fname);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return QString();
    QTextStream f(&file);
    QString txt = f.readAll();
    file.close();
    return txt;
}

void writeFile(QString fname, QString txt) {
    QFile file(fname);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return;
    QTextStream f(&file);
    f << txt;
    file.close();
}

QRect getDesktopRect() {
    int   xmax = 0;
    int   ymax = 0;
    QList<QScreen *> screens = QGuiApplication::screens();
    for (int i = 0; i < screens.count(); i++) {
        QRect r = screens[i]->geometry();
        xmax = qMax(xmax, r.right());
        ymax = qMax(ymax, r.bottom());
    }
    return QRect(0, 0, xmax, ymax);
}

QStringList parseCombinedArgString(const QString &program)
{
    QStringList args;
    QString tmp;
    int quoteCount = 0;
    bool inQuote = false;
    // handle quoting. tokens can be surrounded by double quotes
    // "hello world". three consecutive double quotes represent
    // the quote character itself.
    for (int i = 0; i < program.size(); ++i)
    {
        if (program.at(i) == QLatin1Char('"'))
        {
            ++quoteCount;
            if (quoteCount == 3)
            {
                // third consecutive quote
                quoteCount = 0;
                tmp += program.at(i);
            }
            continue;
        }
        if (quoteCount)
        {
            if (quoteCount == 1)
                inQuote = !inQuote;
            quoteCount = 0;
        }
        if (!inQuote && program.at(i).isSpace())
        {
            if (!tmp.isEmpty())
            {
                args += tmp;
                tmp.clear();
            }
        }
        else
        {
            tmp += program.at(i);
        }
    }
    if (!tmp.isEmpty())
        args += tmp;
    return args;
}
