/* osd.cpp  -- OSD class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      osd.cpp
 * @author    Bruno ANSELME
 * @brief     OSDWindow class definition
 * @details   OSD window. Bypass window manager. Called by OSDEditor.
 */

#include <QScreen>
#include <QPainter>
#include <QShowEvent>
#include <QTimerEvent>
#include "misc.h"
#include "osd.h"

OSDWindow::OSDWindow(t_params *p) :
    QFrame(nullptr, Qt::FramelessWindowHint
           | Qt::WindowStaysOnTopHint
           | Qt::X11BypassWindowManagerHint
           | Qt::WindowDoesNotAcceptFocus
           | Qt::SplashScreen),
    ui(new Ui::OSD),
    message(this),
    resizing(false)
{
    params  = p;
    ui->setupUi(this);
    hide();
    rscreens = getDesktopRect();
    changeOsd(params->osdPosition, params->osdSize);
    timer.setSingleShot(true);
    timer.setInterval(params->osdDuration);
    connect(&timer,     SIGNAL(timeout()),  this,   SLOT(endTimer()));
}

void OSDWindow::mousePressEvent(QMouseEvent *event) {
    offset = pos() - event->globalPos();
    resizing = handle.contains(event->pos());
}

void OSDWindow::mouseMoveEvent(QMouseEvent *event)  {
    if (!resizing) {
        changeOsd(event->globalPos() + offset, size());
    } else {
        QSize  newSize(qMax(event->globalPos().x() - pos().x(), 200), qMax(event->globalPos().y() - pos().y(), 40));
        changeOsd(pos(), newSize);
    }
}

void OSDWindow::changeOsd(QPoint pos, QSize siz) {
    setMaximumSize(siz);
    setMinimumSize(siz);
    if (pos.x() > (rscreens.right() - siz.width()))          // Check if OSD window is out of screen
        pos.setX(rscreens.right() - siz.width());
    if (pos.y() > (rscreens.bottom() - siz.height()))
        pos.setY(rscreens.bottom() - siz.height());
    if (pos.x() < 0) pos.setX(0);
    if (pos.y() < 0) pos.setY(0);
    setGeometry(pos.x(), pos.y(), siz.width(), siz.height());
    handle.setRect(rect().left() + rect().width() - 10, rect().top() + rect().height() - 10, 10, 10);
}

void OSDWindow::paintEvent(QPaintEvent *event) {
    QFrame::paintEvent(event);
    if (!timer.isActive()) {                // Editing OSD position and size;
        QPainter    painter(this);
        painter.fillRect(handle, Qt::darkGray);
    }
}

void OSDWindow::setFlashingText(QString s) {
    if (timer.isActive()) timer.stop();
    ui->lbMessage->setStyleSheet("QLabel { }");
    ui->lbMessage->setText(s.simplified());
    show();
    timer.start();
}

void OSDWindow::setUnknown(QString s) {
    if (timer.isActive()) timer.stop();
    ui->lbMessage->setStyleSheet("QLabel { color: red; }");
    ui->lbMessage->setText(s.simplified());
    show();
    timer.start();
}

void OSDWindow::setText(QString s) {
    ui->lbMessage->setStyleSheet("QLabel { }");
    ui->lbMessage->setText(s.simplified());
    show();
}

void OSDWindow::endTimer() {
    hide();
}
