/* selectset.cpp  -- SelectSet class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      selectset.cpp
 * @author    Bruno ANSELME
 * @brief     SelectSet class definition
 * @details   Commands set selector.
 */

#include <QDir>
#include <QCheckBox>
#include <QMessageBox>
#include <QSettings>
#include <QInputDialog>
#include <QFileDialog>
#include "app_version.h"
#include "selectset.h"
#include "editset.h"
#include "fileviewer.h"
#include "misc.h"
#include "editvocabulary.h"

SelectSet::SelectSet(t_params &p, Comprendo *w) : QDialog(),
    ui(new Ui::dialSelector)
{
    params  = p;
    ui->setupUi(this);

    updateSets();
    connect(ui->btNew,     SIGNAL(clicked()),      this,   SLOT(newSet()));
    connect(ui->btEdit,    SIGNAL(clicked()),      this,   SLOT(editSet()));
    connect(ui->btClone,   SIGNAL(clicked()),      this,   SLOT(cloneSet()));
    connect(ui->btDelete,  SIGNAL(clicked()),      this,   SLOT(deleteSet()));
    connect(ui->btExport,  SIGNAL(clicked()),      this,   SLOT(exportSet()));
    connect(ui->btImport,  SIGNAL(clicked()),      this,   SLOT(importSet()));
    connect(ui->btCsv,     SIGNAL(clicked()),      this,   SLOT(exportCsv()));
    connect(ui->btLM,      SIGNAL(clicked()),      this,   SLOT(viewLMFile()));
    connect(ui->lstSets,   SIGNAL(itemDoubleClicked(QListWidgetItem*)),    this,   SLOT(editSet()));
    connect(ui->lstSets,   SIGNAL(itemChanged(QListWidgetItem *)),         this,   SLOT(transmitSetUpdated()));
    ui->lstSets->setCurrentRow(0);
    what = w;
    QSettings settings(APP_ORG, APP_NAME);
    move(settings.value("selPos", QPoint(300, 300)).toPoint());
    resize(settings.value("selSize", QSize(500, 350)).toSize());
}

SelectSet::~SelectSet() {
    QSettings settings(APP_ORG, APP_NAME);
    settings.setValue("selPos",            pos());
    settings.setValue("selSize",           size());
}

void SelectSet::transmitSetUpdated() {
    writeSettings();
    emit setsUpdated();
}

void SelectSet::updateSets() {
    ui->lstSets->clear();
    QStringList filter = { "*.cmd" };
    QStringList files = params.langPath.entryList(filter);
    QStringList keys = getSetFiles(params.langShort);
    QListIterator<QString> filename(files);
    while (filename.hasNext()) {
        QString name = filename.next();
        name.replace(QRegExp("\\.cmd$"),"");
        QListWidgetItem *item = new QListWidgetItem(name);
        item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
        if (keys.contains(name))
            item->setCheckState(Qt::Checked);
        else
            item->setCheckState(Qt::Unchecked);
        ui->lstSets->addItem(item);
    }
}

QStringList SelectSet::getSetFiles(QString lang) {
    QSettings settings(APP_ORG, APP_NAME);      // Read sets settings here
    settings.beginGroup(lang);
    QStringList keys = settings.allKeys();
    settings.endGroup();
    return keys;
}

void SelectSet::editSet() {
    EditSet dial(params, ui->lstSets->currentItem()->text());
    connect(&dial,  SIGNAL(setUpdated()),   this,   SLOT(transmitSetUpdated()));
    dial.exec();
}

void SelectSet::accept() {
    writeSettings();
    QDialog::accept();
}

void SelectSet::viewLMFile() {
    FileViewer dial(params.lmFile, tr("Language model file"));
    dial.exec();
}

void SelectSet::exportCsv() {
    QString fileName = QFileDialog::getSaveFileName(nullptr, tr("Export commands CSV"), QString("%1.csv").arg(APP_NAME), "*.csv");
    if (QFileInfo(fileName).suffix().isEmpty()) { fileName.append(".csv"); }
    writeFile(fileName, what->getCommandsCSV());
}

void SelectSet::newSet() {
    bool ok;
    QString text = QInputDialog::getText(this, tr("New commands set"),
                                         tr("New commands set name :"), QLineEdit::Normal,
                                         tr("untitled"), &ok);
    if (ok && !text.isEmpty()) {
        QString dstname    = QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(text);
        QFileInfo check_file(dstname);
        if (check_file.exists() && check_file.isFile()) {
            QMessageBox msgBox;
            msgBox.setText(tr("Commands set <b>%1</b> already exists.").arg(text));
            msgBox.exec();
        } else {
            QFile file(dstname);
            if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
                return;
            file.close();
            updateSets();
        }
    }
}

void SelectSet::importSet() {
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                    QString("%1/").arg(QDir::homePath()),
                                                    QString("%1 (*.noc)").arg(APP_NAME));
    if (!fileName.isEmpty()) {
        QFileInfo info(fileName);
        QString txt = readFile(fileName);
        QString dict = readFile(params.dictFile);       // Read language's words
        QRegExp     rword("^\\d+\\s:.*$");              // RegExp for line format matching
        QStringList lines = txt.split("\n");
        txt.clear();
        for (int i = 0; i < lines.count(); i++) {
            if (rword.exactMatch(lines[i])) {
                txt.append(lines[i].append("\n"));      // Add to commands
            } else {
                dict.append(lines[i].append("\n"));     // Add to vocabulary
            }
        }
        EditVocabulary::sortVocabulary(dict);           // Sort, renumber and remove duplicate
        QString dstname    = QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(info.baseName());
        QFileInfo check_file(dstname);
        if (check_file.exists() && check_file.isFile()) {
            QMessageBox msgBox;
            msgBox.setText(tr("Commands set <b>%1</b> already exists.").arg(info.baseName()));
            msgBox.exec();
        } else {
            writeFile(dstname,txt);                     // New commands set
            writeFile(params.dictFile,dict);            // Updated vocabulary
            updateSets();                               // Update this dialog
            emit setsUpdated();                          // Update language model
        }
    }
}

void SelectSet::exportSet() {
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"),
                                                    QString("%1/%2.noc").arg(QDir::homePath()).arg(ui->lstSets->currentItem()->text()),
                                                    QString("%1 (*.noc)").arg(APP_NAME));
    if (!fileName.isEmpty()) {
        CommandSet cmds(QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(ui->lstSets->currentItem()->text()));
        QString utts = cmds.getCommandsUtterance();
        utts.replace(" ","\n");
        QStringList words = utts.split("\n");           // Build words list from commandset utterances
        words.sort();
        words.removeDuplicates();
        QString dict = readFile(params.dictFile);       // Read language's words
        QRegExp     rword("^([\\w\\d]+).*$");           // RegExp for first word matching
        QStringList         parts;
        QString     txt;
        QStringList lines   = dict.split("\n");
        for (int i = 0; i < lines.count(); i++) {
            if (rword.exactMatch(lines[i])) {
                parts  = rword.capturedTexts();
                for (int j=0; j < words.count(); j++) {
                    if (parts[1] == words[j])
                        txt.append(lines[i]).append("\n");
                }
            }
        }
        QFile file(fileName);
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
            return;
        QTextStream f(&file);
        f << txt;
        f << readFile(QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(ui->lstSets->currentItem()->text()));
        file.close();
        QMessageBox msgBox;
        msgBox.setText(tr("Exported in file : <b>%1</b>").arg(fileName));
        msgBox.exec();
    }
}

void SelectSet::cloneSet() {
    bool ok;
    QString text = QInputDialog::getText(this, tr("Clone commands set"),
                                         tr("Clone <b>%1</b> as :").arg(ui->lstSets->currentItem()->text()), QLineEdit::Normal,
                                         QString("%1.clone").arg(ui->lstSets->currentItem()->text()), &ok);
    if (ok && !text.isEmpty()) {
        QString srcname    = QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(ui->lstSets->currentItem()->text());
        QString dstname    = QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(text);
        QFileInfo check_file(dstname);
        if (check_file.exists() && check_file.isFile()) {
            QMessageBox msgBox;
            msgBox.setText(tr("Commands set <b>%1</b> already exists.").arg(text));
            msgBox.exec();
        } else {
            QFile::copy(srcname, dstname);
        }
        updateSets();
    }
}

void SelectSet::deleteSet() {
    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(this,
                                  tr("Delete set"),
                                  tr("Delete set : <b>%1</b>").arg(ui->lstSets->currentItem()->text()),
                                  QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
    if (reply == QMessageBox::Yes) {
        QString srcname    = QString("%1/%2.cmd").arg(params.langPath.absolutePath()).arg(ui->lstSets->currentItem()->text());
        QFile::remove(srcname);
        updateSets();
        emit setsUpdated();
    }
}


void SelectSet::writeSettings() {
    QSettings settings(APP_ORG, APP_NAME);
    settings.remove(params.langShort);
    settings.beginGroup(params.langShort);
    for (int i = 0; i < ui->lstSets->count(); i++) {
        QListWidgetItem *item = ui->lstSets->item(i);
        if (item->checkState() & Qt::Checked) {
            settings.setValue(item->text(), item->text().append(".cmd"));
        }
    }
    settings.endGroup();
}
