/* settings.cpp  -- Settings class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      settings.cpp
 * @author    Bruno ANSELME
 * @brief     Settings class definition
 * @details   Settings dialog for NoComprendo.
 */

#include <QDebug>
#include <iostream>
#include <QSystemTrayIcon>
#include <QSettings>
#include <QMessageBox>
#include "app_version.h"
#include "settings.h"
#include "misc.h"
#include "utterancebuilder.h"
#include "langchooser.h"
#include "guicontrol.h"
#include "speak.h"

Settings::Settings(t_params *p) : QDialog(),
    ui(new Ui::NCSettings)
{
    ui->setupUi(this);
    params      = p;
    QLocale     loc(params->lang);
    ui->ckAutostart->setChecked(params->autoStart);
    ui->ckAsk2Quit->setChecked(params->ask2Quit);
    ui->sprMaxLines->setValue(params->maxlines);
    ui->ckRemoveBadUtterance->setChecked(params->removeBadUtterance);
    ui->ckHideTaskBar->setChecked(params->hideInTaskBar);
    ui->ckSysTray->setChecked(params->useSysTray);
    ui->ckSysTray->setEnabled(QSystemTrayIcon::isSystemTrayAvailable());
    ui->lbLang->setText(tr("Language : %1/%2").arg(loc.nativeLanguageName()).arg(loc.nativeCountryName()));
    ui->ckEspeak->setChecked(params->speak);
    ui->ckOsdOn->setChecked(params->osdOn);
    ui->ckOsdAll->setChecked(params->osdAll);
    ui->sbOsdDuration->setValue(params->osdDuration/1000);
    ui->cbGender->setCurrentIndex(params->gender-1);
    ui->spAge->setValue(params->age);
    ui->spMouseStep->setValue(params->mouseStep);
    ui->spGlideStep->setValue(params->glideStep);
    ui->spGlideRate->setValue(params->glideRate);
    osdPosition = params->osdPosition;
    osdSize     = params->osdSize;
    ui->sprFontSize->setValue(params->rFontSize);
    ui->ckInvertColor->setChecked(params->rInvertColor);
    ui->ckWordsToLM->setChecked(params->wordsToLM);
    connect(ui->btPlay,         SIGNAL(clicked()),                      this,   SLOT(playVoice()));
    connect(ui->btOsdPosition,  SIGNAL(clicked()),                      this,   SLOT(moveOsd()));
    connect(ui->tbMouseCom,     SIGNAL(cellDoubleClicked(int, int)),    this,   SLOT(editUtterance(int, int)));
    connect(ui->btLang,         SIGNAL(clicked()),                      this,   SLOT(changeLanguage()));
    connect(ui->ckHideTaskBar,  SIGNAL(toggled(bool)),                  this,   SLOT(changeTaskBar(bool)));
#ifdef ENABLE_ESPEAK
    // Init cbVoiceVariant here
    QString voices;
    voices = listVoices(params->langShort.toStdString().c_str());
    QStringList elangs = voices.trimmed().split("\n");
    for (int i = 0; i < elangs.count(); i++) {
        QStringList elmts = elangs[i].split(" : ");
        ui->cbVoiceVariant->addItem(elangs[i], elmts[1]);
    }
    if (params->voiceVariant >= elangs.count())
        params->voiceVariant = 0;
    ui->cbVoiceVariant->setCurrentIndex(params->voiceVariant);
#else
    ui->tabEspeak->setEnabled(false);
    ui->tabSettings->removeTab(ui->tabSettings->indexOf(ui->tabEspeak));
#endif
    QString txt =   readFile(params->mouseFile);
    QStringList     vocabulary;
    vocabulary = txt.trimmed().split("\n");
    if (vocabulary.count() == 10) {
        for (int i = 0; i < ui->tbMouseCom->rowCount(); i++) {
            ui->tbMouseCom->item(i,0)->setText(vocabulary[i]);
        }
    }
    changeTaskBar(params->hideInTaskBar);
    QSettings settings(APP_ORG, APP_NAME);
    move(settings.value("confPos", QPoint(100, 100)).toPoint());
    resize(settings.value("confSize", QSize(454, 519)).toSize());
}

Settings::~Settings() {
    QSettings settings(APP_ORG, APP_NAME);
    settings.setValue("confPos",            pos());
    settings.setValue("confSize",           size());
}

void Settings::accept() {
    if (params->hideInTaskBar != ui->ckHideTaskBar->isChecked())
        emit sendInformation(tr("Task bar needs application restart."));
    params->gender              = ui->cbGender->currentIndex()+1;
    params->age                 = ui->spAge->value();
    params->speak               = ui->ckEspeak->isChecked();
    params->autoStart           = ui->ckAutostart->isChecked();
    params->ask2Quit            = ui->ckAsk2Quit->isChecked();
    params->maxlines            = ui->sprMaxLines->value();
    params->removeBadUtterance  = ui->ckRemoveBadUtterance->isChecked();
    params->hideInTaskBar       = ui->ckHideTaskBar->isChecked();
    params->useSysTray          = ui->ckSysTray->isChecked();
    params->mouseStep           = ui->spMouseStep->value();
    params->glideStep           = ui->spGlideStep->value();
    params->glideRate           = ui->spGlideRate->value();
    params->osdOn               = ui->ckOsdOn->isChecked();
    params->osdAll              = ui->ckOsdAll->isChecked();
    params->osdDuration         = ui->sbOsdDuration->value()*1000;
    params->osdPosition         = osdPosition;
    params->osdSize             = osdSize;
    params->rFontSize           = ui->sprFontSize->value();
    params->rInvertColor        = ui->ckInvertColor->isChecked();
    params->wordsToLM           = ui->ckWordsToLM->isChecked();
#ifdef ENABLE_ESPEAK
    params->voiceVariant        = ui->cbVoiceVariant->currentIndex();
#endif
    QString out;        // Save mouse grid utterances
    for (int i = 0; i < ui->tbMouseCom->rowCount(); i++) {
        out.append(ui->tbMouseCom->item(i,0)->text()).append("\n");
    }
    writeFile(params->mouseFile,out);
    QDialog::accept();
}

void Settings::editUtterance(int row, int ) {
    UtteranceBuilder dial(params, ui->tbMouseCom->item(row,0)->text());
    int dialogCode = dial.exec();
    if (dialogCode == QDialog::Accepted) {
        ui->tbMouseCom->item(row,0)->setText(dial.utterance().trimmed());
    }
}

void Settings::changeTaskBar(bool b) {
    if (b) {
        ui->ckSysTray->setChecked(true);
        ui->ckSysTray->setEnabled(false);
    } else {
        ui->ckSysTray->setEnabled(true);
    }
}

void Settings::moveOsd() {
    OSDEditor   osde(params);
    osde.exec();
    osdPosition = osde.getPosition();
    osdSize     = osde.getSize();
}

void Settings::playVoice() {
#ifdef ENABLE_ESPEAK
    setVoice(ui->cbVoiceVariant->currentData().toString().toStdString().c_str(), ui->cbGender->currentIndex()+1, ui->spAge->value(), ui->cbVoiceVariant->currentIndex());
    sayIt(ui->leToSay->text().toStdString().c_str());
#endif
}

void Settings::changeLanguage() {
    QSettings   settings(APP_ORG, APP_NAME);
    LangChooser    dial;                       // Language selector
    int dialogCode = dial.exec();
    if (dialogCode == QDialog::Accepted) {
        QString lang = dial.getLang();
        if (params->lang != lang) {
            params->lang = lang;
            settings.setValue("defLang", params->lang);
            emit sendInformation(tr("Language change will take effect when the application restart."));
        }
    }
}
