/* trayicon.cpp  -- TrayIcon class implementation for NoComprendo */
/*
   Copyright (C) 2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      trayicon.cpp
 * @author    Bruno ANSELME
 * @brief     TrayIcon class definition
 * @details   System tray icon for NoComprendo
 */

#include "app_version.h"
#include "trayicon.h"

TrayIcon::TrayIcon(t_params *p) : QSystemTrayIcon()
{
    params  = p;

    icons[Stopped]          = QIcon(":/icons/nocomprendo-stop.png");
    icons[Ready]            = QIcon(":/icons/nocomprendo-ready.png");
    icons[Listening]        = QIcon(":/icons/nocomprendo-listen.png");
    icons[Suspended]        = QIcon(":/icons/nocomprendo-suspend.png");
    icons[ListenSuspend]    = QIcon(":/icons/nocomprendo-listen-suspend.png");

    messages[Stopped]       = tr("%1 stopped").arg(APP_NAME);
    messages[Ready]         = tr("%1 ready").arg(APP_NAME);
    messages[Listening]     = tr("%1 listening").arg(APP_NAME);
    messages[Suspended]     = tr("%1 suspended").arg(APP_NAME);
    messages[ListenSuspend] = tr("%1 listening").arg(APP_NAME);

    setIcon(icons[Stopped]);

    QAction    *quitAction, *showAction;
    showAction  = myMenu.addAction(tr("Show"));
    myMenu.addSeparator();

    actions[Ready]          = myMenu.addAction(QIcon(":/icons/start.png"), tr("Start"));
    actions[Stopped]        = myMenu.addAction(QIcon(":/icons/stop.png"),  tr("Stop"));
    myMenu.addSeparator();
    actions[Suspended]      = myMenu.addAction(QIcon(":/icons/suspend.png"), tr("Suspend"));
    myMenu.addSeparator();

    quitAction  = myMenu.addAction(QIcon(":/icons/exit.png"), tr("Quit %1").arg(APP_NAME));

    connect(actions[Ready],     SIGNAL(triggered()), this,  SLOT(sendStart()));
    connect(actions[Stopped],   SIGNAL(triggered()), this,  SLOT(sendStop()));
    connect(actions[Suspended], SIGNAL(triggered()), this,  SLOT(sendSuspend()));
    connect(quitAction,         SIGNAL(triggered()), this,  SLOT(sendQuit()));
    connect(showAction,         SIGNAL(triggered()), this,  SLOT(sendShow()));
    setContextMenu(&myMenu);
    show();
}

void TrayIcon::setMenus(bool running, bool suspended) {
    actions[Ready]->setEnabled(!running);
    actions[Stopped]->setEnabled(running);
    actions[Suspended]->setEnabled(running);
    actions[Suspended]->setText((running && !suspended) ? tr("Suspend") : tr("Resume") );
}

void TrayIcon::setStatus(Status s) {
    setIcon(icons[s]);
    setToolTip(messages[s]);
}

void TrayIcon::sendStart()      { emit trayStart();     }
void TrayIcon::sendStop()       { emit trayStop();      }
void TrayIcon::sendSuspend()    { emit traySuspend();   }
void TrayIcon::sendQuit()       { emit trayQuit();      }
void TrayIcon::sendShow()       { emit trayShow();      }

