/* zoombutton.cpp  -- ZoomButton class implementation for NoComprendo */
/*
   Copyright (C) 2019-2020 Bruno ANSELME <be.root@free.fr>

   This file is part of the NoComprendo software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file      zoombutton.cpp
 * @author    Bruno ANSELME
 * @brief     ZoomButton class definition
 * @details   Split window into 9 boxes, 4 times to get a mouse location.
 *              Special button class for Mouse MoveTo command. Based on keypads numbers.
 */

#include <iostream>
#include <QCoreApplication>
#include <QPainter>
#include <QEventLoop>
#include <QKeyEvent>
#include "zoombutton.h"
#include "guicontrol.h"
#include "misc.h"

ZoomButton::ZoomButton(t_params *p, Comprendo *comp) : QAbstractButton(),
    done(false),
    here(0,0),
    offset(0,0)
{
    params  = p;
    setWindowOpacity(0.6);
    setWindowFlags(Qt::WindowStaysOnTopHint);
    frames.append(rect());
    showFullScreen();
    installEventFilter(this);
    what    = comp;
    timerid = startTimer(10);
    int x, y, screen;
    unsigned int w, h;
    screen  = 0;
    while ((screen<2) && (guiScreenSize(&w, &h, screen) == 0)) {
        screens.append(QRect(0, 0, static_cast<int>(w), static_cast<int>(h)));
        screen++;
    }
    guiGetMousePos(&x, &y, &screen);
    // Only two screens expected
    if (x > screens[0].width())     offset += QPoint(screens[0].width(), 0);
    if (y > screens[0].height())    offset += QPoint(0, screens[0].height());

    QString txt =   readFile(params->mouseFile);
    QStringList     vocabulary;
    vocabulary = txt.trimmed().split("\n");
    if (vocabulary.count() == 10) {
        for (int i = 0; i < vocabulary.count(); i++) {
            boxes[vocabulary[i]] = i;
        }
    }
}

void ZoomButton::exec() {
    QEventLoop wait_loop;
    connect(this,   SIGNAL(clicked()),             &wait_loop, SLOT(quit()));
    connect(this,   SIGNAL(focusLost(bool)),       &wait_loop, SLOT(quit()));
    connect(this,   SIGNAL(zoomClicked()),         &wait_loop, SLOT(quit()));
    wait_loop.exec();   // Wait for a click or focus lost or mouse move finished
 }

bool ZoomButton::eventFilter(QObject *, QEvent *event)
{
   if (event->type() == QEvent::WindowDeactivate)
      emit focusLost(true);
   return false;
}

void ZoomButton::timerEvent(QTimerEvent *t){
    if ((what) && (t->timerId()==timerid)) {
        QString *word = what->recognize_from_microphone();
        if (!word->isEmpty()) {
            if (params->wordTrace) {
                std::cout << word->toStdString().c_str() << "\n";
                std::cout.flush();
            }
            if (boxes.contains(*word)) {
                pushFrame(boxes[*word]);
            }
        } else
            what->doCommand(*word, false);
        delete word;        // Word created in recognize_from_microphone must be deleted
    }
}

void ZoomButton::pushFrame(int num) {
    if (!num) {         // Box 0 is backward
        popFrame();
        return;
    }
    QRect last = frames.last();
    QRect r = rectNumber(last, num);
    if (frames.count()<4) {
        frames.append(r);
        update();
    } else {
        here = QPoint(r.center())+offset;
        done = true;
        emit zoomClicked();
    }
}

void ZoomButton::popFrame() {
    frames.removeLast();
    if (frames.isEmpty())
        emit clicked();
    else update();
}

void ZoomButton::keyReleaseEvent(QKeyEvent *e) {
    int code = e->key();
    if (code == Qt::Key_Escape)                         popFrame();
    if ((code >= Qt::Key_1) && (code <= Qt::Key_9))     pushFrame(code - Qt::Key_0);
}

void ZoomButton::resizeEvent(QResizeEvent *e) {
    frames.clear();
    frames.append(QRect(QPoint(0,0) ,e->size()));
}

const QRect ZoomButton::rectNumber(QRect r, int num) {
    int w = r.width()/3;
    int h = r.height()/3;
    num--;
//    return QRect((num % 3)*w + r.x(), ((8 - num) / 3)*h + r.y(), w, h);     // Keypad order
    return QRect((num % 3)*w + r.x(), (num / 3)*h + r.y(), w, h);           // Natural order, left to right, top to bottom
}

void ZoomButton::drawGrid(QRect r, QPainter *painter, int level) {
    int h = r.height()/3;
    QFont font = painter->font();
    font.setPixelSize(2*h/3);
    painter->setFont(font);
    for(int count=1 ; count<10; count++)
        painter->drawRect(rectNumber(r, count));
    if (level == (frames.count()-1))
        for(int count=1 ; count<10; count++)
            painter->drawText(rectNumber(r, count), Qt::AlignHCenter|Qt::AlignVCenter, QString("%1").arg(count));
}

void ZoomButton::paintEvent(QPaintEvent *) {
    QPainter    painter(this);
    painter.setPen(Qt::black);
    painter.fillRect(rect(), Qt::darkGray);
    for(int i = 0; i < frames.count(); i++) {
        drawGrid(frames[i], &painter, i);
    }
}
